# encoding: utf-8

'''Pygubu-designer Virtual Events'''

FILE_NEW = '<<ACTION_FILE_NEW>>'
FILE_OPEN = '<<ACTION_FILE_OPEN>>'
FILE_SAVE = '<<ACTION_FILE_SAVE>>'
FILE_SAVEAS = '<<ACTION_FILE_SAVEAS>>'
FILE_QUIT = '<<ACTION_FILE_QUIT>>'
FILE_RECENT_CLEAR = '<<ACTION_FILE_RECENT_CLEAR>>'
TREE_NAV_UP = '<<ACTION_TREE_NAV_UP>>'
TREE_NAV_DOWN = '<<ACTION_TREE_NAV_DOWN>>'
TREE_ITEM_MOVE_DOWN = '<<ACTION_TREE_ITEM_MOVE_DOWN>>'
TREE_ITEM_MOVE_UP = '<<ACTION_TREE_ITEM_MOVE_UP>>'
TREE_ITEM_COPY = '<<ACTION_TREE_ITEM_COPY>>'
TREE_ITEM_PASTE = '<<ACTION_TREE_ITEM_PASTE>>'
TREE_ITEM_CUT = '<<ACTION_TREE_ITEM_CUT>>'
TREE_ITEM_DELETE = '<<ACTION_TREE_ITEM_DELETE>>'
TREE_ITEM_GRID_UP = '<<ACTION_TREE_ITEM_GRID_UP>>'
TREE_ITEM_GRID_DOWN = '<<ACTION_TREE_ITEM_GRID_DOWN>>'
TREE_ITEM_GRID_LEFT = '<<ACTION_TREE_ITEM_GRID_LEFT>>'
TREE_ITEM_GRID_RIGHT = '<<ACTION_TREE_ITEM_GRID_RIGHT>>'
TREE_ITEM_PREVIEW_TOPLEVEL = '<<ACTION_TREE_ITEM_PREVIEW_TOPLEVEL>>'
PREVIEW_TOPLEVEL_CLOSE_ALL = '<<ACTION_PREVIEW_TOPLEVEL_CLOSE_ALL>>'




# encoding: UTF-8
#
# Copyright 2012-2017 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
import re

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubudesigner.widgets.propertyeditor import (ChoicePropertyEditor,
                                                   register_editor)


class TtkStylePropertyEditor(ChoicePropertyEditor):
    RE_TTKSTYLECLASS = re.compile('(?i)$|[_A-Za-z](\.[_a-zA-Z0-9]+|[_a-zA-Z0-9]*)*$')
    
    def _validate(self):
        valid = False
        value = self._get_value()
        m = self.RE_TTKSTYLECLASS.match(value)
        if m:
            valid = True
        self.show_invalid(not valid)
        return valid


register_editor('ttkstylechoice', TtkStylePropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()
    root.columnconfigure(0, weight=1)
    root.rowconfigure(0, weight=1)

    def make_on_change_cb(editor):
        def on_change_cb(event=None):
            print(editor.value)
            print(repr(editor.value))
        return on_change_cb

    editor = TtkStylePropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('mystyle.TButton')
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    root.mainloop()



#file: nuitkahelper

#Help nuitka compiler to include specific modules
import pygubu.builder.tkstdwidgets
import pygubu.builder.ttkstdwidgets
import pygubu.builder.widgets.dialog
import pygubu.builder.widgets.editabletreeview
import pygubu.builder.widgets.scrollbarhelper
import pygubu.builder.widgets.scrolledframe
import pygubu.builder.widgets.tkscrollbarhelper
import pygubu.builder.widgets.tkscrolledframe
import pygubu.builder.widgets.pathchooserinput



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
try:
    import tkinter.ttk as ttk
except:
    import ttk

from pygubudesigner.widgets.propertyeditor import *


class WHPropertyEditor(PropertyEditor):

    def _create_ui(self):
        self._wlabel = w = ttk.Label(self, text='w:',
                                     font='TkSmallCaptionFont')
        w.grid(row=0, column=0)
        self._weditor = w = EntryPropertyEditor(self)
        w.grid(row=0, column=1, sticky='we')
        w.parameters(width=4)
        
        self._wlabel = w = ttk.Label(self, text='h:',
                                     font='TkSmallCaptionFont')
        w.grid(row=0, column=2)
        self._heditor = w = EntryPropertyEditor(self)
        w.grid(row=0, column=3, sticky='we')
        w.parameters(width=4)

        self._weditor.bind('<<PropertyChanged>>', self._on_variable_changed)
        self._heditor.bind('<<PropertyChanged>>', self._on_variable_changed)

        self.rowconfigure(0, weight=1)
        self.columnconfigure(1, weight=1)
        self.columnconfigure(3, weight=1)

    def _get_value(self):
        value = ''
        if self._weditor.value != '' and self._heditor.value != '':
            value = '{0}|{1}'.format(self._weditor.value, self._heditor.value)
        return value

    def _set_value(self, value):
        if '|' in value:
            width, heigth = value.split('|')
            self._weditor.edit(width)
            self._heditor.edit(heigth)
        else:
            self._weditor.edit('')
            self._heditor.edit('')
    
    def _validate(self):
        isvalid = False
        w = self._weditor.value
        h = self._heditor.value
        if w == '' and h == '':
            isvalid = True
        else:
            try:
                if int(w) >= 0 and int(h) >= 0:
                    isvalid = True
            except ValueError:
                pass
        self.show_invalid(not isvalid)
        return isvalid

register_editor('whentry', WHPropertyEditor)



#helloworld.py
import tkinter as tk
import pygubu


class HelloWorldApp:
    
    def __init__(self):

        #1: Create a builder
        self.builder = builder = pygubu.Builder()

        #2: Load an ui file
        builder.add_from_file('holamundo.ui')

        #3: Create the mainwindow
        self.mainwindow = builder.get_object('mainwindow')
        
    def run(self):
        self.mainwindow.mainloop()


if __name__ == '__main__':
    app = HelloWorldApp()
    app.run()


# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals

__all__ = ['PropertyEditor', 'EntryPropertyEditor', 'SpinboxPropertyEditor',
           'ChoicePropertyEditor', 'ChoiceByKeyPropertyEditor',
           'TextPropertyEditor', 'CheckbuttonPropertyEditor',
           'register_editor', 'create_editor']

import keyword
import re

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubu.widgets.scrollbarhelper import ScrollbarHelper
from pygubu.widgets.combobox import Combobox

EDITORS = {}
KEY_PRESS_CB_MILISECONDS = 500


def register_editor(name, class_):
    EDITORS[name] = class_


def create_editor(name, *args, **kw):
    editor = EDITORS[name](*args, **kw)
    return editor


class PropertyEditor(ttk.Frame):
    style_initialized = False
    
    def __init__(self, master=None, **kw):
        self._variable = tk.StringVar()
        self._initvalue = None
        self._cbid = None # callback id
        self.value = ''
        ttk.Frame.__init__(self, master, **kw)
        
        if not PropertyEditor.style_initialized:
            s = ttk.Style()
            s.configure('PropertyEditorInvalid.TFrame', background='red')
        self.configure(borderwidth=2)
        self._create_ui()
        
    def _on_keypress(self, event=None):
        if self._cbid is not None:
            self.after_cancel(self._cbid)
        self._cbid = self.after(KEY_PRESS_CB_MILISECONDS, self._on_keypress_after)
    
    def _on_keypress_after(self, event=None):
        self._on_variable_changed(event)
        self._cbid = None
        
    def show_invalid(self, invalid=True):
        if invalid:
            self.configure(style='PropertyEditorInvalid.TFrame')
        else:
            self.configure(style='TFrame')

    def _create_ui(self):
        pass

    def _validate(self):
        return True

    def _get_value(self):
        """Get value from storage"""
        return self._variable.get()

    def _set_value(self, value):
        """Save value on storage"""
        self._variable.set(value)

    def _on_variable_changed(self, event=None):
        if self._validate():
            self.value = self._get_value()
            if self.value != self._initvalue:
                self.event_generate('<<PropertyChanged>>')
                self._initvalue = self.value
                self._after_change()

    def _after_change(self):
        pass
    
    def parameters(self, **kw):
        pass

    def edit(self, value):
        self._initvalue = value
        self.value = value
        self._set_value(value)


class EntryPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._entry = entry = ttk.Entry(self, textvariable=self._variable)
        entry.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        entry.bind('<FocusOut>', self._on_variable_changed)
        entry.bind('<KeyPress>', self._on_keypress)

    def parameters(self, **kw):
        self._entry.configure(**kw)


class AlphanumericEntryPropertyEditor(EntryPropertyEditor):
    def _validate(self):
        is_valid = True
        value = self._get_value()
        if len(value) != 0:
            is_valid = value.isalnum()
        self.show_invalid(not is_valid)
        return is_valid


class IdentifierPropertyEditor(EntryPropertyEditor, object):
    RE_IDENTIFIER = re.compile('[_A-Za-z][_a-zA-Z0-9]*$')
    
    def __init__(self, master=None, **kw):
        self.is_unique_cb = kw.pop('unique_cb', None)
        super(IdentifierPropertyEditor, self).__init__(master, **kw)
    
    def set_unique_cb(self, callback):
        self.is_unique_cb = callback
    
    def _validate(self):
        is_valid = True
        value = self._get_value()
        if len(value) > 0:
            if keyword.iskeyword(value):
                is_valid = False
            if is_valid and not self.RE_IDENTIFIER.match(value):
                is_valid = False
            # Check if new id is unique
            if is_valid and self.is_unique_cb is not None:
                if value != self._initvalue:
                    is_valid = self.is_unique_cb(value)
        else:
            # ID must have at least one character
            is_valid = False
        self.show_invalid(not is_valid)
        return is_valid


SpinboxClass = tk.Spinbox
if tk.TkVersion >= 8.6 and not hasattr(ttk, 'Spinbox'):
    from pygubu.widgets.ttkspinbox import Spinbox
    SpinboxClass = Spinbox


class SpinboxPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._spinbox = SpinboxClass(self, textvariable=self._variable,
                                     command=self._on_variable_changed)
        spinbox = self._spinbox
        spinbox.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        spinbox.bind('<FocusOut>', self._on_variable_changed)
        spinbox.bind('<KeyPress>', self._on_keypress)

    def parameters(self, **kw):
        self._spinbox.configure(**kw)


class TextPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._sbh = ScrollbarHelper(self)
        self._sbh.grid(row=0, column=0, sticky='we')
        self._text = text = tk.Text(self._sbh.container, width=20, height=3)
        self._sbh.add_child(self._text)
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        text.bind('<FocusOut>', self._on_variable_changed)
        text.bind('<KeyPress>', self._on_keypress)

    def _get_value(self):
        value = self._text.get('0.0', tk.END)
        if value.endswith('\n'):
            value = value[:-1]
        return value

    def _set_value(self, value):
        self._text.delete('0.0', tk.END)
        self._text.insert('0.0', value)


class ChoicePropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._cb_pending = False
        self._combobox = combobox = ttk.Combobox(self,
                                                 textvariable=self._variable)
        combobox.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        combobox.bind('<FocusOut>', self._on_variable_changed)
        combobox.bind('<<ComboboxSelected>>', self._on_variable_changed)
        combobox.bind('<KeyPress>', self._on_keypress)

    def parameters(self, **kw):
        self._combobox.configure(**kw)


class ChoiceByKeyPropertyEditor(ChoicePropertyEditor):
    def _create_ui(self):
        self._cb_pending = False
        self._combobox = combobox = Combobox(self, keyvariable=self._variable)
        combobox.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        combobox.bind('<FocusOut>', self._on_variable_changed)
        combobox.bind('<<ComboboxSelected>>', self._on_variable_changed)
        combobox.bind('<KeyPress>', self._on_keypress)


class CheckbuttonPropertyEditor(PropertyEditor):
    def _create_ui(self):
        self._checkb = w = ttk.Checkbutton(self, variable=self._variable)
        w.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        w.configure(command=self._on_variable_changed)

    def parameters(self, **kw):
        self._checkb.configure(**kw)


class NaturalNumberEditor(EntryPropertyEditor):
    def _validate(self):
        valid = True
        value = self._get_value()
        if len(value) != 0:
            valid = False
            try:
                number = int(value)
                if number >= 0:
                    valid = True
            except:
                pass
        self.show_invalid(not valid)
        return valid


class IntegerNumberEditor(EntryPropertyEditor):
    def _validate(self):
        valid = True
        value = self._get_value()
        if len(value) != 0:
            valid = False
            try:
                int(value)
                valid = True
            except:
                pass
        self.show_invalid(not valid)
        return valid


class RealNumberEditor(EntryPropertyEditor):
    def _validate(self):
        valid = True
        value = self._get_value()
        if len(value) != 0:
            valid = False
            try:
                float(value)
                valid = True
            except:
                pass
        self.show_invalid(not valid)
        return valid


class GeometryPropertyEditor(ChoicePropertyEditor):
    RE_GEOMETRY = re.compile('\d+x\d+$')
    
    def _validate(self):
        is_valid = True
        value = self._get_value()
        if len(value) != 0:
            if not self.RE_GEOMETRY.match(value):
                is_valid = False
        self.show_invalid(not is_valid)
        return is_valid


class LayoutManagerPropertyEditor(ChoicePropertyEditor):
    """Special Internal Manager property editor"""
    
    def _create_ui(self):
        self._cb_pending = False
        self._combobox = combobox = Combobox(self, keyvariable=self._variable)
        combobox.grid(sticky='we')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        combobox.bind('<FocusOut>', self._on_variable_changed)
        combobox.bind('<<ComboboxSelected>>', self._on_variable_changed)
        combobox.bind('<KeyPress>', self._on_keypress)


register_editor('entry', EntryPropertyEditor)
register_editor('alphanumentry', AlphanumericEntryPropertyEditor)
register_editor('identifierentry', IdentifierPropertyEditor)
register_editor('choice', ChoicePropertyEditor)
register_editor('choice_key', ChoiceByKeyPropertyEditor)
register_editor('spinbox', SpinboxPropertyEditor)
register_editor('text', TextPropertyEditor)
register_editor('checkbutton', CheckbuttonPropertyEditor)
register_editor('naturalnumber', NaturalNumberEditor)
register_editor('integernumber', IntegerNumberEditor)
register_editor('realnumber', RealNumberEditor)
register_editor('geometryentry', GeometryPropertyEditor)



if __name__ == '__main__':
    root = tk.Tk()
    root.columnconfigure(0, weight=1)
    root.rowconfigure(0, weight=1)

    def make_on_change_cb(editor):
        def on_change_cb(event=None):
            print(editor.value)
            print(repr(editor.value))
        return on_change_cb

    editor = EntryPropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('MyValue')
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    editor = TextPropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('MyValue 2')
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    editor = SpinboxPropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('55')
    editor.parameters(from_=0, to=100, increment=2)
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    editor = ChoicePropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('')
    editor.parameters(values=('A', 'B', 'C', 'D'))
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))

    root.mainloop()



import os
import pygubu
import pathlib

PROJECT_PATH = pathlib.Path(__file__).parent
PROJECT_UI = PROJECT_PATH / "demoapp.ui"


class DemoApp:
    def __init__(self):
        self.builder = builder = pygubu.Builder()
        builder.add_resource_path(PROJECT_PATH)
        print(PROJECT_PATH)
        builder.add_from_file(PROJECT_UI)
        self.mainwindow = builder.get_object('topdemo')
        builder.connect_callbacks(self)
        
    def run(self):
        self.mainwindow.mainloop()

if __name__ == '__main__':
    app = DemoApp()
    app.run()




try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk


class ToolbarFrame(ttk.Frame):
    BTN_LEFT_STYLE = 'BtnLeft.ToolbarFrame.Toolbutton'
    BTN_RIGHT_STYLE = 'btnRight.ToolbarFrame.Toolbutton'
    
    def __init__(self, master=None, **kw):
        ttk.Frame.__init__(self, master, **kw)
        #btonbar = ttk.Frame(master)
        fvport = ttk.Frame(self)
        fcontent = ttk.Frame(fvport)
        fcontent.config(height='50', width='30', padding=0)
        fcontent.place(anchor='nw', x='0', y='0', bordermode='inside',
                       relheight=1)
        fvport.config(height='50', width='30')
        fvport.pack(expand='true', fill='both', side='left')
        fcontrols = ttk.Frame(self)
        bsleft = ttk.Button(fcontrols)
        bsleft.config(text='<', width='1', takefocus=True,
                      style=self.BTN_LEFT_STYLE)
        bsleft.pack(expand='true', fill='y', side='left')
        bsleft.configure(command=self.scroll_left)
        bsright = ttk.Button(fcontrols)
        bsright.config(text='>', width='1', takefocus=True,
                      style=self.BTN_RIGHT_STYLE)
        bsright.pack(expand='true', fill='y', side='left')
        bsright.configure(command=self.scroll_right)
        fcontrols.config(height='50', width='50')
        fcontrols.pack(fill='y', side='left')
        #btonbar.config(height='50', width='320')
        #btonbar.pack(expand='true', fill='x', side='top')
        self.config(height='200', width='20')
        
        self.controls_visible = True
        self.controls_required = False
        self._scrollRecurse = 0
        self.timer = None
        self.fcontrols = fcontrols
        self.fvport = fvport
        self.fcontent = fcontent
        self.fcstart = 0
        self.SCROLL_INCREMENT = 50
        fvport.bind('<Configure>', self._reposition)
        fcontent.bind('<Configure>', self._reposition)
    
    def child_master(self):
        return self.fcontent
    
    def toggle_controls(self):
        self.controls_visible = not self.controls_visible
        if self.controls_visible:
            self.fcontrols.pack(side='left', fill='y')
        else:
            self.fcontrols.pack_forget()
    
    def reposition(self):
        if self.timer is None:
            self.timer = self.after_idle(self._reposition)
    
    def _reposition(self, event):
        self.timer = None

        # Call update_idletasks to make sure that the containing frame
        # has been resized before we attempt to set the scrollbars.
        # Otherwise the scrollbars may be mapped/unmapped continuously.
        self._scrollRecurse = self._scrollRecurse + 1
        self.update_idletasks()
        self._scrollRecurse = self._scrollRecurse - 1
        if self._scrollRecurse != 0:
            return
        
        fcw = self.fcontent.winfo_reqwidth()
        #fch = self.fcontent.winfo_reqheight()
        fctlw = self.fcontrols.winfo_reqwidth()
        #fctlh = self.fcontrols.winfo_reqheight()
        myw = self.winfo_width()
        #myh = self.winfo_height()
        vpw = self.fvport.winfo_width()
        #vph = self.fvport.winfo_height()
        
        self.SCROLL_INCREMENT = vpw//3
        hole = myw - fctlw
        diff = hole - fcw
        
        if diff < 0:
            # requires controls
            self.controls_required = True
        else:
            # remove controls
            self.controls_required = False
            if self.fcstart != 0:
                self.fcstart = 0
                self.fcontent.place(x=self.fcstart)
        
        if self.controls_required != self.controls_visible:
            self.toggle_controls()

    def scroll_right(self):
        newstart = self.fcstart - self.SCROLL_INCREMENT
        cw = self.fcontent.winfo_reqwidth()
        limit = cw - self.SCROLL_INCREMENT
        if newstart > -(limit):
            self.fcstart = newstart
            self.fcontent.place(x=self.fcstart)

    def scroll_left(self):
        newstart = self.fcstart + self.SCROLL_INCREMENT
        if newstart <= 0:
            self.fcstart = newstart
            self.fcontent.place(x=self.fcstart)


if __name__ == '__main__':
    root = tk.Tk()
    widget = ToolbarFrame(root)
    widget.pack(expand=True, fill='both')
    root.mainloop()





import pathlib
import random
import pygubu


PROJECT_PATH = pathlib.Path(__file__).parent
PROJECT_UI = PROJECT_PATH / "userinput.ui"


class UserinputApp:
    def __init__(self):
        self.builder = builder = pygubu.Builder()
        builder.add_resource_path(PROJECT_PATH)
        builder.add_from_file(PROJECT_UI)
        self.mainwindow = builder.get_object('mainwindow')
        builder.connect_callbacks(self)

        # Get from builder all variables we need
        # variables will be accesible from self
        guivars = ('entryvar', 'spinvar', 'combovar',
                   'option1var', 'option2var', 'option3var',
                   'group1var')
        builder.import_variables(self, guivars)

        # Fill with random values
        self.random_values()

    def random_values(self):
        # Entry
        txt = 'Random string '
        rand = random.sample('0123456789abcdefghijklmnoprstuvwxyz', 10)
        txt = txt + ''.join(rand)
        self.entryvar.set(txt)

        # Spinbox
        val = random.randint(0, 100)
        self.spinvar.set(val)

        # combobox
        combo = self.builder.get_object('combobox')
        options = combo.cget('values')
        selection = random.choice(options)
        self.combovar.set(selection)

        # Checkbuttons
        for var in (self.option1var, self.option2var, self.option3var):
            value = 0 if random.random() < 0.5 else 1
            var.set(value)

        # Radiobuttons
        value = random.choice(('A', 'B', 'C'))
        self.group1var.set(value)

    def on_change_clicked(self):
        self.random_values()

    def on_print_clicked(self):
        line = f'Entry value:{self.entryvar.get()}'
        print(line)
        line = f'Spinbox value:{self.spinvar.get()}'
        print(line)
        line = f'Checkbox Option1 value:{self.option1var.get()}'
        print(line)
        line = f'Checkbox Option2 value:{self.option2var.get()}'
        print(line)
        line = f'Checkbox Option3 value:{self.option3var.get()}'
        print(line)
        line = f'Radiobutton Group value:{self.group1var.get()}'
        print(line)

    def run(self):
        self.mainwindow.mainloop()

if __name__ == '__main__':
    app = UserinputApp()
    app.run()




# encoding: UTF-8
#
# Copyright 2012-2017 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
import re

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubudesigner.widgets.propertyeditor import *

re_dim = '\d+([cimp])?'
regexp = '({0})?$'.format(re_dim)
RE_DIMENSION = re.compile(regexp)
regexp = '({0})?$|{0}\s{0}$'.format(re_dim)
RE_TWO_DIMENSION = re.compile(regexp)
regexp = '({0})?$|{0}\s{0}$|{0}\s{0}\s{0}\s{0}$'.format(re_dim)
RE_FOUR_DIMENSION = re.compile(regexp)


class DimensionPropertyEditor(EntryPropertyEditor):
    REGEX = RE_DIMENSION
    def __init__(self, master=None, **kw):
        self._empty_data = None
        EntryPropertyEditor.__init__(self, master, **kw)
    
    def _validate(self):
        valid = False
        value = self._get_value()
        m = self.REGEX.match(value)
        if m:
            valid = True
        self.show_invalid(not valid)
        return valid
    
    def _get_value(self):
        value = self._variable.get()
        
        if self._empty_data is not None and value == '':
            value = str(self._empty_data)
            self._set_value(value)
        
        return value
    
    def parameters(self, **kw):
        pvalue = kw.pop('empty_data', None)
        self._empty_data = None if pvalue is None else pvalue
        EntryPropertyEditor.parameters(self, **kw)


class TwoDimensionPropertyEditor(DimensionPropertyEditor):
    REGEX = RE_TWO_DIMENSION


class FourDimensionPropertyEditor(DimensionPropertyEditor):
    REGEX = RE_FOUR_DIMENSION


register_editor('dimensionentry', DimensionPropertyEditor)
register_editor('twodimensionentry', TwoDimensionPropertyEditor)
register_editor('fourdimensionentry', FourDimensionPropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()
    root.columnconfigure(0, weight=1)
    root.rowconfigure(0, weight=1)

    def make_on_change_cb(editor):
        def on_change_cb(event=None):
            print(editor.value)
            print(repr(editor.value))
        return on_change_cb

    editor = DimensionPropertyEditor(root)
    editor.pack(expand=True, fill='x')
    editor.edit('10m')
    editor.bind('<<PropertyChanged>>', make_on_change_cb(editor))
    
    editor2 = TwoDimensionPropertyEditor(root)
    editor2.pack(expand=True, fill='x')
    editor2.edit('10p 20p')
    editor2.bind('<<PropertyChanged>>', make_on_change_cb(editor2))    
    
    editor2 = FourDimensionPropertyEditor(root)
    editor2.pack(expand=True, fill='x')
    editor2.edit('10p 20p 1m 2m')
    editor2.bind('<<PropertyChanged>>', make_on_change_cb(editor2))        

    root.mainloop()



import os
from .preferences import recent_files_get, recent_files_save


class RecentFilesManager(object):
    def __init__(self, menu, itemcallback):
        self.menu = menu
        self.item_callback = itemcallback
        self.filelist = []
        
    def load(self):
        self.filelist = recent_files_get()
        for f in self.filelist:
            self._add_item(f)
    
    def save(self):
        newlist = self.filelist[:10]
        self.filelist = newlist
        recent_files_save(reversed(newlist))
    
    def addfile(self, filepath):
        if filepath not in self.filelist:
            self.filelist.insert(0, filepath)
            self._add_item(filepath)
        else:
            self.filelist.remove(filepath)
            self.filelist.insert(0, filepath)
        self.save()
    
    def _add_item(self, filepath):
        name = os.path.basename(filepath)
        itemlabel = '{0} [{1}]'.format(name, filepath)
        
        def item_command(fname=filepath, cb=self.item_callback):
            cb(filepath)
        
        self.menu.insert_command(0, label=itemlabel, command=item_command)
    
    def clear(self):
        count = self.menu.index('end')
        index_to = count - 2
        if index_to > 0:
            self.menu.delete(0, index_to)
        self.filelist = []
        self.save()





# encoding: utf-8

# translator marker
def _(x):
    return x

tooltip_help = {
    'class': _('Object class name.'),
    'id': _('Object unique identifier.'),
    'accelerator': _('Specifies a string to display at the right side of the menu entry.'),
    'activerelief': _('Specifies the relief to use when displaying the element that is active, if any.'),
    'activestyle': _('Specifies the style in which to draw the active element.'),
    'activebackground': _('Specifies background color to use when drawing active elements.'),
    'activeborderwidth': _('Specifies a non-negative value indicating the width of the 3-D border drawn around active elements.'),
    'activeforeground': _('Specifies foreground color to use when drawing active elements.'),
    'after': None,
    'anchor': _('Specifies how the information in the widget is positioned relative to the inner margins.'),
    'aspect': _('Specifies a non-negative integer value indicating desired aspect ratio for the text. The aspect ratio is specified as 100*width/height. 100 means the text should be as wide as it is tall, 200 means the text should be twice as wide as it is tall, 50 means the text should be twice as tall as it is wide, and so on.'),
    'autoseparators': _('Specifies a boolean that says whether separators are automatically inserted in the undo stack. Only meaningful when the undo option is true.'),
    'background-tk': _("Specifies the normal background color to use when displaying the widget."),
    'background-ttk': _("The widget's background color. If unspecified, the theme default is used."),
    'borderwidth': _('Specifies a non-negative value indicating the width of the 3-D border to draw around the outside of the widget.'),
    'bigincrement': _('Some interactions with the scale cause its value to change by “large” increments; this option specifies the size of the large increments.'),
    'bitmap': _('Specifies a bitmap to display in the widget.'),
    'blockcursor': _('Specifies a boolean that says whether the blinking insertion cursor should be drawn as a character-sized rectangular block.'),
    'buttonbackground': _("The widget's background color. If unspecified, the theme default is used."),
    'buttonbackground-tk.Spinbox': _('The background color to be used for the spin buttons.'),
    'buttoncursor': _('The cursor to be used when over the spin buttons.'),
    'buttondownrelief': _('The relief to be used for the upper spin button.'),
    'buttonuprelief': _('The relief to be used for the lower spin button.'),
    'class_': _("Specifies the window class. The class is used when querying the option database for the window's other options, to determine the default bindtags for the window, and to select the widget's default layout and style."),
    'closeenough': _('Specifies a floating-point value indicating how close the mouse cursor must be to an item before it is considered to be “inside” the item. Defaults to 1.0.'),
    'column_anchor': _('Specifies how the text in this column should be aligned with respect to the cell.'),
    'columnbreak': _('When this option is False, the entry appears below the previous entry. When this option is True, the entry appears at the top of a new column in the menu. This option is ignored on Aqua/macOS, where menus are always a single column.'),
    'command': None,
    'command-pygubu': _('In pygubu-designer, a python function name used as the callback for the widget command property.'),
    'compound-tk': _('Specifies if the widget should display text and bitmaps/images at the same time, and if so, where the bitmap/image should be placed relative to the text.'),
    'compound-ttk': _('Specifies how to display the image relative to the text, in the case both text and image are present.'),
    'confine': _("Specifies a boolean value that indicates whether or not it should be allowable to set the canvas's view outside the region defined by the scrollRegion argument."),
    'container': _('If true, it means that this window will be used as a container in which some other application will be embedded. The window should not have any children of its own in this application.'),
    'cursor': _('Specifies the mouse cursor to be used for the widget.'),
    'default': _("In a dialog box, one button may be designated the “default” button. active indicates that this is currently the default button; normal means that it may become the default button, and disabled means that it is not defaultable."),
    'digits': _('An integer specifying how many significant digits should be retained when converting the value of the scale to a string.'),
    'direction': _('Specifies where the menu is to be popped up relative to the menubutton.'),
    'disabledbackground': _('Specifies the background color to use when the widget is disabled.'),
    'disabledforeground': _('Specifies foreground color to use when drawing a disabled element.'),
    'elementborderwidth': _('Specifies the width of borders drawn around the internal elements of the scrollbar (the two arrows and the slider).'),
    'endline': _('Specifies an integer line index representing the line of the underlying textual data store that should be just after the last line contained in the widget. This allows a text widget to reflect only a portion of a larger piece of text.'),
    'exportselection': _('Specifies whether or not a selection in the widget should be linked to the X selection.'),
    'font': _('Specifies the font to use when drawing text inside the widget.'),
    'foreground-tk': _('Specifies the normal foreground color to use when displaying the widget.'),
    'foreground-ttk': _("The widget's foreground color. If unspecified, the theme default is used."),
    'format': _('Specifies an alternate format to use when setting the string value when using the from and to range. This must be a format specifier of the form %<pad>.<pad>f, as it will format a floating-point number.'),
    'from_': _('A floating-point value specifying the lowest value for the widget.'),
    'handlepad': _('When sash handles are drawn, specifies the distance from the top or left end of the sash (depending on the orientation of the widget) at which to draw the handle.'),
    'handlesize': _('Specifies the side length of a sash handle. Handles are always drawn as squares.'),
    'heading_anchor': _('Specifies how the heading text should be aligned.'),
    'height': _('Specifies the desired height for the widget.'),
    'height-tk.Button': _('Specifies a desired height for the button. If an image or bitmap is being displayed in the button then the value is in screen units; for text it is in lines of text.'),
    'height-tk.Text': _('Specifies the desired height for the window, in units of characters in the font given by the font option.'),
    'height-tk.Menubutton': _('Specifies a desired height for the menubutton. If an image or bitmap is being displayed in the menubutton then the value is in screen units; for text it is in lines of text.'),
    'height-ttk.Treeview': _('Specifies the number of rows which should be visible.'),
    'height-ttk.Combobox': _('Specifies the height of the pop-down listbox, in rows.'),
    'hidemargin': _('Specifies whether the standard margins should be drawn for this menu entry. This is useful when creating palette with images in them.'),
    'highlightbackground': _('Specifies the color to display in the traversal highlight region when the widget does not have the input focus.'),
    'highlightcolor': _('Specifies the color to use for the traversal highlight rectangle that is drawn around the widget when it has the input focus.'),
    'highlightthickness': _('Specifies a non-negative value indicating the width of the highlight rectangle to draw around the outside of the widget when it has the input focus.'),
    'image-tk': _('Specifies an image to display in the widget'),
    'image-ttk': _('Specifies an image to display. This is a list of 1 or more elements. The first element is the default image name.'),
    'image-ttk.Treeview.Column': _('Specifies an image to display to the right of the column heading.'),
    'inactiveselectbackground': _('Specifies the colour to use for the selection (the sel tag) when the window does not have the input focus.'),
    'increment': _('A floating-point value specifying the increment.'),
    'indicatoron': _('Specifies whether or not the indicator should be drawn.'),
    'insertbackground': _('Specifies the color to use as background in the area covered by the insertion cursor.'),
    'insertborderwidth': _('Specifies a non-negative value indicating the width of the 3-D border to draw around the insertion cursor.'),
    'insertofftime': _('Specifies a non-negative integer value indicating the number of milliseconds the insertion cursor should remain “off” in each blink cycle.'),
    'insertontime': _('Specifies a non-negative integer value indicating the number of milliseconds the insertion cursor should remain “on” in each blink cycle.'),
    'insertunfocussed': _('Specifies how to display the insertion cursor when the widget does not have the focus.'),
    'insertwidth': _('Specifies a value indicating the total width of the insertion cursor.'),
    'invalidcommand': None,
    'jump': _("For widgets with a slider that can be dragged to adjust a value, such as scrollbars, this option determines when notifications are made about changes in the value.If the value is false, updates are made continuously as the slider is dragged. If the value is true, updates are delayed until the mouse button is released to end the drag; at that point a single notification is made."),
    'justify': _('If there are multiple lines of text, specifies how the lines are laid out relative to one another.'),
    'justify-ttk.Entry': _('Specifies how the text is aligned within the entry widget.'),
    'label': _('A string to display as a label for the scale.'),
    'labelanchor': _('Specifies where to place the label. The default value is theme-dependent.'),
    'length': _('Specifies the desired long dimension of the scale in screen units.'),
    'listvariable': _('A variable name to be used as source of items for the Listbox'),
    'maximum': _('A floating point number specifying the maximum value. Defaults to 100.'),
    'maxundo': _('Specifies the maximum number of compound undo actions on the undo stack. A zero or a negative value imply an unlimited undo stack.'),
    'minsize': _('Specifies that the size of the window cannot be made less than n. This constraint only affects the size of the widget in the paned dimension'),
    'minwidth': _('The minimum width of the column in pixels. The treeview widget will not make the column any smaller than minwidth when the widget is resized or the user drags a column separator.'),
    'mode': None,
    'offrelief': _('Specifies the relief for the checkbutton when the indicator is not drawn and the checkbutton is off. The default value is “raised”.'),
    'offvalue': _('The value to store in the associated variable when the widget is deselected. Defaults to 0.'),
    'onvalue': _('The value to store in the associated variable when the widget is selected. Defaults to 1.'),
    'opaqueresize': _('Specifies whether panes should be resized as a sash is moved (true), or if resizing should be deferred until the sash is placed (false). In the latter case, a “ghost” version of the sash is displayed during the resizing to show where the panes will be resized to when releasing the mouse button. This “ghost” version of the sash is the proxy.'),
    'orient': _('For widgets that can lay themselves out with either a horizontal or vertical orientation, this option specifies which orientation should be used.'),
    'overrelief': _('Specifies an alternative relief for the button, to be used when the mouse cursor is over the widget.'),
    'padding': _('Specifies the internal padding for the widget. The padding is a list of up to four length specifications left top right bottom.'),
    'padx': _('Specifies a non-negative value indicating how much extra space to request for the widget in the X-direction.'),
    'pady': _('Specifies a non-negative value indicating how much extra space to request for the widget in the Y-direction.'),
    'postcommand': _('In pygubu-designer, a python function name used as the callback for the widget postcommand property. The function is executed immediately before displaying the listbox. The postcommand callback may specify the values to display.'),
    'proxybackground': _('Background color to use when drawing the proxy.'),
    'proxyborderwidth': _('Specifies the borderwidth of the proxy.'),
    'proxyrelief': _('Relief to use when drawing the proxy. May be any of the standard Tk relief values.'),
    'readonlybackground': _('Specifies the background color to use when the entry is readonly.'),
    'relief': _('Specifies the 3-D effect desired for the widget border.'),
    'repeatdelay': _('Specifies the number of milliseconds a button or key must be held down before it begins to auto-repeat.'),
    'repeatinterval': _('Used in conjunction with repeatdelay: once auto-repeat begins, this option determines the number of milliseconds between auto-repeats.'),
    'resolution': _('A real value specifying the resolution for the scale.'),
    'sliderlength': _("Specifies the size of the slider, measured in screen units along the slider's long dimension."),
    'sliderrelief': _('Specifies the relief to use when drawing the slider.'),
    'sashcursor': _('Mouse cursor to use when over a sash.'),
    'sashpad': _('Specifies the amount of padding to leave of each side of a sash.'),
    'sashrelief': _('Relief to use when drawing a sash.'),
    'sashwidth': _('Specifies the width of each sash.'),
    'selectbackground': _('Specifies the background color to use when displaying selected items.'),
    'selectborderwidth': _('Specifies a non-negative value indicating the width of the 3-D border to draw around selected items.'),
    'selectforeground': _('Specifies the foreground color to use when displaying selected items.'),
    'scrollregion': _('Specifies a list with four coordinates describing the left, top, right, and bottom coordinates of a rectangular region.'),
    'selectcolor': _('Specifies a background color to use when the button is selected.'),
    'selectimage': _('Specifies an image to display (in place of the image option) when the checkbutton is selected. This option is ignored unless the image option has been specified.'),
    'selectmode': _('Controls how the built-in class bindings manage the selection.'),
    'setgrid': _('Specifies a boolean value that determines whether this widget controls the resizing grid for its top-level window.'),
    'show': _("If this option is specified, then the true contents of the entry are not displayed in the window. Instead, each character in the entry's value will be displayed as the first character in the value of this option."),
    'show-ttk.Treeview': _('Specifies which elements of the tree to display.'),
    'showhandle': _('Specifies whether sash handles should be shown.'),
    'showvalue': _('Specifies a boolean value indicating whether or not the current value of the scale is to be displayed.'),
    'spacing1': _('Requests additional space above each text line in the widget, using any of the standard forms for screen distances.  If a line wraps, this option only applies to the first line on the display.'),
    'spacing2': _('For lines that wrap (so that they cover more than one line on the display) this option specifies additional space to provide between the display lines that represent a single line of text.'),
    'spacing3': _('Requests additional space below each text line in the widget, using any of the standard forms for screen distances. If a line wraps, this option only applies to the last line on the display.'),
    'startline': _('Specifies an integer line index representing the first line of the underlying textual data store that should be contained in the widget. This allows a text widget to reflect only a portion of a larger piece of text.'),
    'state': _('May be set to normal or disabled to control the disabled state bit.'),
    'state-tk.Checkbutton': _('Specifies one of three states for the checkbutton: normal, active, or disabled.'),
    'state-tk.Canvas': _('Modifies the default state of the canvas where state may be set to one of: normal, disabled, or hidden. Individual canvas objects all have their own state option which may override the default state.'),
    'sticky': _("Specifies how the content window is positioned within the pane area."),
    'stretch': _('Controls how extra space is allocated to each of the panes.'),
    'stretch-ttk.Treeview': _('Specifies whether or not the column width should be adjusted when the widget is resized or the user drags a column separator.'),
    'style': _('May be used to specify a custom widget style.'),
    'tabs': _("Specifies a set of tab stops for the window. The option's value consists of a list of screen distances giving the positions of the tab stops, each of which is a distance relative to the left edge of the widget (excluding borders, padding, etc)."),
    'tabstyle': _('Specifies how to interpret the relationship between tab stops on a line and tabs in the text of that line.'),
    'takefocus': _('Determines whether the window accepts the focus during keyboard traversal.'),
    'tearoff': _('Specifies whether or not the menu should include a tear-off entry at the top.'),
    'tearoffcommand': _('Specifies a function callback to invoke whenever the menu is torn off.'),
    'text': _('Specifies a text string to be displayed inside the widget.'),
    'text-custom': _('Custom option to set the widget text.'),
    'textvariable': _('Specifies the name of a variable whose value will be used in place of the text resource.'),
    'tickinterval': _('Must be a real value. Determines the spacing between numerical tick marks displayed below or to the left of the slider.'),
    'title-menu': 'The string will be used to title the window created when this menu is torn off. If the title is NULL, then the window will have the title of the menubutton or the text of the cascade item from which this menu was invoked.',
    'to': _('A floating-point value specifying the highest permissible value for the widget.'),
    'tristateimage': _('Specifies an image to display (in place of the image option) when the checkbutton is in tri-state mode. This option is ignored unless the image option has been specified.'),
    'tristatevalue': _('Specifies the value that causes the checkbutton to display the multi-value selection, also known as the tri-state mode. Defaults to “”.'),
    'troughcolor': _('Specifies the color to use for the rectangular trough areas in widgets such as scrollbars and scales.'),
    'underline': _('Specifies the integer index (0-based) of a character to underline in the text string.'),
    'undo': _('Specifies a boolean that says whether the undo mechanism is active or not.'),
    'value': _('Specifies the current value of the widget.'),
    'values': _('Specifies the list of values to display in the drop-down listbox.'),
    'values-tk.Spinbox': _('Must be a proper list value. If specified, the spinbox will use these values as to control its contents, starting with the first value. This option has precedence over the from and to range.'),
    'validate': _('Specifies the mode in which validation should operate.'),
    'validatecommand': None,
    'variable': _('The name of a variable whose value is linked to the widget.'),
    'weight': _('An integer specifying the relative stretchability of the pane. When the paned window is resized, the extra space is added or subtracted to each pane proportionally to its weight.'),
    'width': _('Specifies the desired width for the widget.'),
    'width-tk': _('Specifies a desired width for the label. If an image or bitmap is being displayed in the label then the value is in screen units; for text it is in characters.'),
    'width-ttk': _('If greater than zero, specifies how much space, in character widths, to allocate for the text label. If less than zero, specifies a minimum width.'),
    'width-tk.Label': _('Specifies a desired width for the label. If an image or bitmap is being displayed in the label then the value is in screen units; for text it is in characters.'),
    'width-tk.Button': _('Specifies a desired width for the button. If an image or bitmap is being displayed in the button then the value is in screen units. For a text button (no image or with compound none) then the width specifies how much space in characters to allocate for the text label. If the width is negative then this specifies a minimum width.'),
    'width-tk.Entry': _("Specifies an integer value indicating the desired width of the entry window, in average-size characters of the widget's font."),
    'width-tk.Spinbox': _("Specifies an integer value indicating the desired width of the spinbox window, in average-size characters of the widget's font. If the value is less than or equal to zero, the widget picks a size just large enough to hold its current text."),
    'width-tk.Message': _('Specifies the length of lines in the window. If this option has a value greater than zero then the aspect option is ignored and the width option determines the line length. If this option has a value less than or equal to zero, then the aspect option determines the line length.'),
    'width-tk.Menubutton': _("Specifies a desired width for the menubutton. If an image or bitmap is being displayed in the menubutton then the value is in screen units; for text it is in characters. If this option is not specified, the menubutton's desired width is computed from the size of the image or bitmap or text being displayed in it."),
    'width-tk.Text': _('Specifies the desired width for the window in units of characters in the font given by the font option.'),
    'width-ttk.Label': _('If greater than zero, specifies how much space, in character widths, to allocate for the text label. If less than zero, specifies a minimum width.'),
    'width-tk.Scrollbar': _('Specifies the desired narrow dimension of the scrollbar window, not including 3-D border, if any. For vertical scrollbars this will be the width and for horizontal scrollbars this will be the height.'),
    'width-scale': _("Specifies the desired narrow dimension of the scale in screen units.  For vertical scales this is the scale's width; for horizontal scales this is the scale's height."),
    'width-ttk.Treeview.Column': _('The width of the column in pixels. Default is 200 pixels.'),
    'wrap-ttk.Spinbox': _('If on, the spinbox will wrap around the values of data in the widget.'),
    'wrap-tk.Text': _("Specifies how to handle lines in the text that are too long to be displayed in a single line of the text's window."),
    'wraplength': _('Specifies the maximum line length (in pixels).'),
    'xscrollcommand': None,
    'xscrollincrement': _('Specifies an increment for horizontal scrolling, in any of the usual forms permitted for screen distances.'),
    'yscrollcommand': None,
    'yscrollincrement': _('Specifies an increment for vertical scrolling, in any of the usual forms permitted for screen distances.'),
    #
    # Custom properties
    'geometry-custom': _("Set the window geometry. Allowed string format: 'wxh'"),
    'maxsize-custom': _('Set the maximum window size.'),
    'minsize-custom': _('Set the minimum window size.'),
    'overrideredirect-custom': _('Sets the override redirect flag. If True, removes all window manager decorations from the window, so that it cannot be moved, resized, iconified, or closed.'),
    'title-custom': _('Sets the window title.'),
    'tree_column-custom': _('If True, set this column as the tree column #0'),
    'visible-custom': _('Determines whether or not the column is included in the displaycolumns list.'),
    'resizable-custom': _('Determines if the window can be resized.'),
    'iconbitmap-custom': _('Sets bitmap for the iconified widget.'),
    'iconphoto-custom': _('Sets the titlebar icon for this window.'),
    'scrolltype-custom': _('Determines scroll type for the widget.'),
    'usermosewheel': _('Enable scrolling using mousewheel'),
    #
    # Layout properties
    #
    'padx-layout': _('Specifies how much horizontal external padding to leave on each side of the content.'),
    'pady-layout': _('Specifies how much vertical external padding to leave on each side of the content.'),
    'ipadx-layout': _('Specifies how much horizontal internal padding to leave on each side of the content.'),
    'ipady-layout': _('Specifies how much vertical internal padding to leave on each side of the content.'),
    'propagate-layout': _('Enable/Disable Geometry Propagation'),
    'anchor-layout': _('It specifies where to position each content in its parcel.'),
    'anchor-layout-place': _('Specifies which point of window is to be positioned at the (x,y) location selected by the x, y, relx, and rely options.'),
    # pack properties
    'expand-pack': _('Specifies whether the content should be expanded to consume extra space in their container.'),
    'fill-pack': _("If a content's parcel is larger than its requested dimensions, this option may be used to stretch the content."),
    'side-pack': _('Specifies which side of the container the content will be packed against.'),
    'bordermode-pack': _('Determines the degree to which borders within the container are used in determining the placement of the content.'),
    'height-pack': _('Specifies the height for window in screen units'),
    'relheight-pack': _('Specifies the height for window. In this case the height is specified as a floating-point number relative to the height of the container: 0.5 means window will be half as high as the container, 1.0 means window will have the same height as the container, and so on. If both height and relheight are specified for a content, their values are summed.'),
    'relwidth-pack': _('Specifies the width for window. In this case the width is specified as a floating-point number relative to the width of the container: 0.5 means window will be half as wide as the container, 1.0 means window will have the same width as the container, and so on. If both width and relwidth are specified for a content, their values are summed.'),
    'relx-pack': _('Specifies the x-coordinate within the container window of the anchor point for window. In this case the location is specified in a relative fashion as a floating-point number: 0.0 corresponds to the left edge of the container and 1.0 corresponds to the right edge of the container.'),
    'rely-pack': _('Specifies the y-coordinate within the container window of the anchor point for window. In this case the value is specified in a relative fashion as a floating-point number: 0.0 corresponds to the top edge of the container and 1.0 corresponds to the bottom edge of the container.'),
    'width-pack': _('Specifies the width for window in screen units.'),
    'x-pack': _('Specifies the x-coordinate within the container window of the anchor point for window.'),
    'y-pack': _('Specifies the y-coordinate within the container window of the anchor point for window.'),
    #
    # grid packing properties
    'row-grid': _('Insert the content so that it occupies the nth row in the grid.'),
    'column-grid': _('Insert the window so that it occupies the nth column in the grid.'),
    'sticky-grid': _("If a content's cell is larger than its requested dimensions, this option may be used to position (or stretch) the content within its cell."),
    'rowspan-grid': _('Insert the content so that it occupies n rows in the grid.'),
    'columnspan-grid': _('Insert the window so that it occupies n columns in the grid.'),
    #
    # grid row and column properties (can be applied to each row or column)
    'minsize-grid': _('Sets the minimum size, in screen units, that will be permitted for this row.'),
    'pad-grid': _('Specifies the number of screen units that will be added to the largest window contained completely in that row when the grid geometry manager requests a size from the containing window.'),
    'weight-grid': _('Sets the relative weight for apportioning any extra spaces among rows.'),
    'uniform-grid': _('When a non-empty value is supplied, places the row in a uniform group with other rows that have the same value for uniform.')
}

def help_for(pname):
    return tooltip_help[pname]




#file: myapp.py
import os
import random
import math

import tkinter as tk
import tkinter.ttk as ttk
import pygubu

PROJECT_PATH = os.path.dirname(__file__)
PROJECT_UI = os.path.join(PROJECT_PATH, "myapp.ui")
DEG2RAD = 4 * math.atan(1) * 2 / 360


class MyApplication:

    def __init__(self):
        self.builder = builder = pygubu.Builder()
        builder.add_resource_path(PROJECT_PATH)
        builder.add_from_file(PROJECT_UI)
        self.mainwindow = builder.get_object('mainwindow')
        
        self.canvas = builder.get_object('main_canvas')
        # Connect to Delete event
        self.mainwindow.protocol("WM_DELETE_WINDOW", self.quit)
        
        builder.connect_callbacks(self)
    
    def btn_draw_clicked(self):
        figures = ['square', 'cross', 'circle', 'triangle']
        figure = random.choice(figures)
        self._draw_figure(figure)
        
        # Update Canvas scroll region
        bbox = self.canvas.bbox('all')
        self.canvas.configure(scrollregion=bbox)
        
    def quit(self, event=None):
        self.mainwindow.quit()

    def run(self):
        self.mainwindow.mainloop()
    
    def _draw_figure(self, figure):
        canvas = self.canvas
        max_iterations = random.randint(3, 10)
        
        for i in range(0, max_iterations):
            canvasw = canvas.winfo_width()
            canvash = canvas.winfo_height()
            borderw = random.randint(1, 5)
            max_width = int(canvas.winfo_width()*0.25)
            w = random.randint(20, max_width)
            x = random.randint(0, canvasw) - int(max_width * 0.5)
            y = random.randint(0, canvash) - int(max_width * 0.5)
            start = random.randint(0, 180)
            
            if figure == 'circle':
                canvas.create_oval(x, y, x+w, y+w, outline='#FF6666', width=borderw)
            if figure == 'square':
                self.create_regpoly(x, y, x+w, y+w,
                                    sides=4, start=start,
                                    outline='#ED9DE9', width=borderw, fill='')
            if figure == 'triangle':
                self.create_regpoly(x, y, x+w, y+w,
                                    sides=3, start=start,
                                    outline='#40E2A0', width=borderw, fill='')
            if figure == 'cross':
                self.create_regpoly(x, y, x+w, y+w,
                                    sides=2, start=start,
                                    outline='#80B3E7', width=borderw, fill='')
                self.create_regpoly(x, y, x+w, y+w,
                                    sides=2, start=start+90,
                                    outline='#80B3E7', width=borderw, fill='')
                
    def create_regpoly(self, x0, y0, x1, y1, sides=0, start=90, extent=360, **kw):
        """Create a regular polygon"""
        coords = self.__regpoly_coords(x0, y0, x1, y1, sides, start, extent)
        return self.canvas.create_polygon(*coords, **kw)

    def __regpoly_coords(self, x0, y0, x1, y1, sides, start, extent):
        """Create the coordinates of the regular polygon specified"""

        coords = []
        if extent == 0:
            return coords

        xm = (x0 + x1) / 2.
        ym = (y0 + y1) / 2.
        rx = xm - x0
        ry = ym - y0

        n = sides
        if n == 0: # 0 sides => circle
            n = round((rx + ry) * .5)
            if n < 2:
                n = 4

        # Extent can be negative
        dirv = 1 if extent > 0 else -1
        if abs(extent) > 360:
            extent = dirv * abs(extent) % 360

        step = dirv * 360 / n
        numsteps = 1 + extent / float(step)
        numsteps_int = int(numsteps)

        i = 0
        while i < numsteps_int:
            rad = (start - i * step) * DEG2RAD
            x = rx * math.cos(rad)
            y = ry * math.sin(rad)
            coords.append((xm+x, ym-y))
            i += 1

        # Figure out where last segment should end
        if numsteps != numsteps_int:
            # Vecter V1 is last drawn vertext (x,y) from above
            # Vector V2 is the edge of the polygon
            rad2 = (start - numsteps_int * step) * DEG2RAD
            x2 = rx * math.cos(rad2) - x
            y2 = ry * math.sin(rad2) - y

            # Vector V3 is unit vector in direction we end at
            rad3 = (start - extent) * DEG2RAD
            x3 = math.cos(rad3)
            y3 = math.sin(rad3)

            # Find where V3 crosses V1+V2 => find j s.t.  V1 + kV2 = jV3
            j = (x*y2 - x2*y) / (x3*y2 - x2*y3)

            coords.append((xm + j * x3, ym - j * y3))

        return coords


if __name__ == '__main__':
    app = MyApplication()
    app.run();




# encoding: UTF-8
#
# Copyright 2012-2014 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubu import builder

CLASS_MAP = builder.CLASS_MAP


class BindingsEditor:
    def __init__(self, etreeview, parent):
        self.tv = etreeview
        self._curr_data = None
        self._curr_bo = None
        self._adder = 'adder'
        self._allow_edit = False
        self._parent = parent
        self.tv.insert('', tk.END, iid=self._adder, values=('+',))
        self.tv.bind('<<TreeviewInplaceEdit>>', self._on_inplace_edit)
        self.tv.bind('<<TreeviewCellEdited>>', self._on_cell_edited)
        self.tv.bind('<Double-Button-1>', self._on_add_clicked, add=True)

        self._del_btn = ttk.Button(self.tv, text='-',
                                   command=self._on_del_clicked)
        self._seq_cboxvar = tk.StringVar()
        self._seq_cbox = cbox = ttk.Combobox(self.tv,
                                             textvariable=self._seq_cboxvar)
        #cbox.place(x=-10, y=-10)
        self.hide_all()

    def _on_add_clicked(self, event):
#        print('_on_add_clicked')
        sel = self.tv.selection()
        if sel:
            item = sel[0]
            if item == self._adder and self._allow_edit:
                self._add_binding(('<1>', 'callback', ''))
                
                # A new binding was added (with default values).
                # Notify the designer that the project has been
                # modified and needs to be saved.
                # Date changed: August 17, 2021
                self._copy_to_data()

    def _on_del_clicked(self):
        sel = self.tv.selection()
        if sel:
            item = sel[0]
            self.tv.delete(item)
        self._copy_to_data()

    def _on_cell_edited(self, event):
        col, item = self.tv.get_event_info()
        self._copy_to_data()

    def _copy_to_data(self):
        if self._curr_data is not None:
            items = self.tv.get_children()
            self._curr_data.clear_bindings()
            for item in items:
                if item != self._adder:
                    values = self.tv.item(item, 'values')
#                    print(item, 'values = ', values)
                    self._curr_data.add_binding(*values[:3])
                    
            # Notify the editor that a change has occurred and the project needs saving.
            # Changed: August 17, 2021        
            self._curr_data.notify(self)

    def _add_binding(self, bind):
        idx = self.tv.index(self._adder)
        self.tv.insert('', idx, values=bind)
        self.tv.selection_set('')

    def edit(self, wdescr):
        wclass = wdescr.classname
        self.clear()
        self._curr_data = wdescr
        self._curr_bo = CLASS_MAP[wclass].builder
        self._seq_cbox.config(values=self._curr_bo.virtual_events)
        
        self._allow_edit = self._curr_bo.allow_bindings
        if self._allow_edit:
            self._parent.pack(fill='both', expand='True')
        else:
            self._parent.pack_forget()
        
        for bind in wdescr.bindings:
            self._add_binding(bind)

    def _on_inplace_edit(self, event):
        col, item = self.tv.get_event_info()
        if item != self._adder and self._allow_edit:
            if col == 'sequence':
                self.tv.inplace_custom(col, item, self._seq_cbox,
                                       self._seq_cboxvar)
            elif col == 'handler':
                self.tv.inplace_entry(col, item)
            elif col == 'add':
                self.tv.inplace_checkbutton(col, item, offvalue='')
            elif col == 'actions':
                self.tv.inplace_custom(col, item, self._del_btn)
    
    def hide_all(self):
        self._parent.pack_forget()

    def clear(self):
        children = self.tv.get_children()
        for item in children:
            if item != self._adder:
                self.tv.delete(item)



try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk
import pygubu.widgets.simpletooltip as tooltip
from .toolbarframe import ToolbarFrame


class ComponentPalette(ttk.Frame):
    def __init__(self, master=None, **kw):
        ttk.Frame.__init__(self, master, **kw)
        component_pallete = ttk.Frame(master)
        fbuttons = ttk.Frame(component_pallete)
        fbuttons.configure(padding=1)
        rb_tk = ttk.Radiobutton(fbuttons)
        self.gvalue = gvalue = tk.StringVar(value='')
        rb_tk.config(style='Toolbutton', text='tk', value='tk',
                     variable=gvalue, command=self._on_tk_clicked)
        rb_tk.pack(expand='true', fill='both', side='top')
        rb_ttk = ttk.Radiobutton(fbuttons)
        rb_ttk.config(style='Toolbutton', text='ttk', value='ttk',
                      variable=gvalue, command=self._on_ttk_clicked)
        rb_ttk.pack(expand='true', fill='both', side='top')
        fbuttons.pack(fill='y', side='left')
        fbntab = ttk.Frame(component_pallete)
        self.notebook = notebook = ttk.Notebook(fbntab)
        notebook.config(height='50', width='300',
                        style='ComponentPalette.TNotebook',
                        takefocus=True)
        notebook.pack(side='top', expand=True, fill='x')
        fbntab.config(height='200', width='200')
        fbntab.pack(side='left', expand=True, fill='x')
        component_pallete.config(height='200', padding='2', width='200')
        component_pallete.pack(side='top', expand=True, fill='x')
        
        self._tabs = {}
        self._buttons = []
        
    def _on_tk_clicked(self):
        self.show_group('tk')
        
    def _on_ttk_clicked(self):
        self.show_group('ttk')
    
    def add_tab(self, tabid, label):
        #frame_1 = ttk.Frame(self.notebook)
        frame_1 = ToolbarFrame(self.notebook)
        frame_1.configure(padding=2)
        frame_1.pack(expand='true', fill='both', side='top')
        self.notebook.add(frame_1, text=label)
        self._tabs[tabid] = frame_1
    
    def add_button(self, tabid, group, label, ttiplabel, image, callback):
        master = self._tabs[tabid].child_master()
        #master = self._tabs[tabid]
        b = ttk.Button(master, text=label, image=image,
                           style='ComponentPalette.Toolbutton', command=callback,
                           compound='top', takefocus=True)
        tooltip.create(b, ttiplabel)
        b.pack(side='left')
        self._buttons.append((b, group))
    
    def show_group(self, group):
        for b, g in self._buttons:
            if g == group:
                b.pack(side='left')
            else:
                b.pack_forget()
        self.gvalue.set(group)


if __name__ == '__main__':
    root = tk.Tk()
    widget = ComponentPallete(root)
    widget.pack(expand=True, fill='both')
    root.mainloop()





# encoding: utf8
from __future__ import unicode_literals
import sys
import os

try:
    import tkinter as tk
    from tkinter import messagebox
except:
    import Tkinter as tk
    import tkMessageBox as messagebox

sys.path.append(os.path.join(os.path.dirname(__file__), '../'))

import pygubu


class Myapp:
    def __init__(self, master):
        self.master = master
        self.builder = builder = pygubu.Builder()
        fpath = os.path.join(os.path.dirname(__file__),"commands.ui")
        builder.add_from_file(fpath)

        mainwindow = builder.get_object('mainwindow', master)

        builder.connect_callbacks(self)
        self.set_scrollbars()

    def on_button_clicked(self):
        messagebox.showinfo('From callback', 'Button clicked !!')

    def validate_number(self, P):
        print('On validate number')
        value = unicode(P)
        return value == '' or value.isnumeric()

    def entry_invalid(self):
        messagebox.showinfo('Title', 'Invalid entry input')

    def radiobutton_command(self):
        messagebox.showinfo('Title', 'Radiobutton command')

    def checkbutton_command(self):
        messagebox.showinfo('Title', 'Checkbutton command')

    def on_scale1_changed(self, event):
        label = self.builder.get_object('scale1label')
        scale = self.builder.get_object('scale1')
        label.configure(text=scale.get())

    def on_scale2_changed(self, event):
        label = self.builder.get_object('scale2label')
        scale = self.builder.get_object('scale2')
        label.configure(text=int(scale.get()))

    def set_scrollbars(self):
        sb1 = self.builder.get_object('scrollbar_1')
        sb2 = self.builder.get_object('scrollbar_2')
        sb1.set(.0, .20)
        sb2.set(.0, .20)

    def on_scrollbar1_changed(self, *args):
        label = self.builder.get_object('scrollbar1label')
        label.configure(text=repr(args))

    def on_scrollbar2_changed(self, *args):
        label = self.builder.get_object('scrollbar2label')
        label.configure(text=repr(args))

    def on_spinbox_cmd(self):
        def showmessage():
            messagebox.showinfo('Title', 'Spinbox command')
        self.master.after_idle(showmessage)

    def on_combobox_validate(self, P):
        value = unicode(P)
        return value == '' or value.isnumeric()

    def on_combobox_invalid(self, P):
        messagebox.showinfo('Title', 'Invalid combobox input')

    def on_combobox_postcmd(self):
        messagebox.showinfo('Title', 'Combobox postcommand')

    def on_menuitem_clicked(self, itemid):
        messagebox.showinfo('Title',
            'Menu item "{0}" was clicked'.format(itemid))

    def on_menuitem2_clicked(self):
        messagebox.showinfo('Title', 'Callback on_menuitem2_clicked')

if __name__ == '__main__':
    root = tk.Tk()
    app = Myapp(root)
    root.mainloop()




# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
import re
try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubudesigner.widgets.compoundpropertyeditor import CompoundPropertyEditor

DIMENSION = r'[1-9]\d*[cimp]?'
DIMENSION_RE = re.compile(DIMENSION)


class SizeEntry(CompoundPropertyEditor):

    def __init__(self, master=None, **kw):
        CompoundPropertyEditor.__init__(self, master, **kw)
        self._mode = ''
        self._dvar = tk.StringVar()
        self._dentry = w = ttk.Entry(self, textvariable=self._dvar)
        w.bind('<FocusOut>', self._on_entry_changed)
        w.bind('<KeyPress>', self._on_keypress)
        w.grid(row=0, column=0, sticky='ew')

        self._whframe = f = ttk.Frame(self)
        f.grid(row=1, column=0, sticky='ew')
        f.columnconfigure(1, weight=1)
        f.columnconfigure(3, weight=1)
        l = ttk.Label(f, text='w:')
        l.grid(row=0, column=0)
        self._wvar = tk.StringVar()
        self._wentry = w = ttk.Entry(f, textvariable=self._wvar, width=9)
        w.grid(row=0, column=1, sticky='ew')
        w.bind('<FocusOut>', self._on_entry_changed)
        w.bind('<KeyPress>', self._on_keypress)

        l = ttk.Label(f, text='h:')
        l.grid(row=0, column=2)
        self._hvar = tk.StringVar()
        self._hentry = w = ttk.Entry(f, textvariable=self._hvar, width=9)
        w.grid(row=0, column='3', sticky='ew')
        w.bind('<FocusOut>', self._on_entry_changed)
        w.bind('<KeyPress>', self._on_keypress)
        self.set_mode()


    def set_mode(self, mode=''):
        self._mode = mode
        if mode == '':
            self._dentry.grid()
            self._whframe.grid_remove()
        if mode == 'whsize':
            self._dentry.grid_remove()
            self._whframe.grid()


    def _on_entry_changed(self, event=None):
        if self._mode == '':
            dvalue = self._dvar.get()
            self._change_value(dvalue)

        elif self._mode == 'whsize':
            wvalue = self._wvar.get()
            hvalue = self._hvar.get()
            valid = False
            empty = False
            if wvalue and hvalue and wvalue.isnumeric() and hvalue.isnumeric():
                valid = True
            elif wvalue == hvalue == '':
                valid = True
                empty = True
            if valid:
                newvalue = ''
                if not empty:
                    newvalue = '{0}|{1}'.format(wvalue, hvalue)
                self._change_value(newvalue)

    def _on_variable_changed(self, varname, elementname, mode):
        newvalue = self._variable.get()
        self._change_value(newvalue, gen_event=False)

    def _change_value(self, newvalue, gen_event=True):
        self._disable_cb()
        if self._mode == '':
            self._variable.set(newvalue)
        elif self._mode == 'whsize':
            w = h = ''
            if '|' in newvalue:
                w, h = newvalue.split('|')
                self._wvar.set(w)
                self._hvar.set(h)
                self._variable.set(newvalue)
            else:
                self._variable.set('')
        self._enable_cb()
        if gen_event:
            self.event_generate('<<SizeEntryChanged>>')


if __name__ == '__main__':
    root = tk.Tk()
    var = tk.StringVar()

    def see_var():
        print(var.get())

    entry = DimensionEntry(root, textvariable=var)
    entry.grid()
    entry.configure(textvariable=var)
    #entry.set_mode('whsize')
    var.set('100|70')

    btn = ttk.Button(root, text='Value', command=see_var)
    btn.grid(row=0, column=1)

    root.mainloop()



from __future__ import unicode_literals, print_function
import os
from collections import OrderedDict
from pygubu.builder import Builder, CLASS_MAP, CB_TYPES
from pygubu.builder.builderobject import BuilderObject, grouper, register_widget
from pygubu.builder.widgetmeta import WidgetMeta
from pygubu.stockimage import TK_BITMAP_FORMATS
from pygubu.builder.tkstdwidgets import TKToplevel


class ToplevelOrTk(TKToplevel):
    def code_realize(self, boparent, code_identifier=None):
        if code_identifier is not None:
            self._code_identifier = code_identifier
        lines = []
        master = boparent.code_child_master()
        init_args = self._get_init_args()
        bag = []
        for pname, value in init_args.items():
            s = "{0}='{1}'".format(pname, value)
            bag.append(s)
        kwargs = ''
        if bag:
            kwargs = ', {0}'.format(', '.join(bag))
        if init_args:
            # has init args defined so create a TkToplevel
            s = "{0} = {1}({2}{3})"
        else:
            s = "{0} = tk.Tk() if master is None else {1}({2}{3})"
        s = s.format(self.code_identifier(), self._code_class_name(),
                     master, kwargs)
        lines.append(s)
        return lines


register_widget('pygubudesigner.ToplevelOrTk',
                ToplevelOrTk, 'ToplevelOrTk', tuple())



class UI2Code(Builder):
    def __init__(self):
        super(UI2Code, self).__init__()
        
        self.buffer = None
        self.as_class = False
        self._code_imports = OrderedDict()
        self._tkvariables = {}
        self._tkimages = {}
        self._callbacks = {}
        self._import_ttk = True;
        self._code = []
        self.uidefinition = None
        self._builder = Builder()
        self._options = {}
    
    def _process_options(self, kw):
        kwdef = {
            'as_class': True,
            'tabspaces': 8
            }
        kwdef.update(kw)
        self._options = kwdef
        self.as_class = self._options['as_class']
        tabspaces = self._options['tabspaces']
        
    def _process_results(self, target):
        tabspaces = self._options['tabspaces']
        code = []
        for line in self._code:
            line = '{0}{1}\n'.format(' ' * tabspaces, line)
            code.append(line)
        code = ''.join(code)
        
        code_imports = self._process_imports()
        code_imports = '\n'.join(code_imports)
        code_callbacks = self._process_callbacks()
        code_callbacks = '\n'.join(code_callbacks)
        code = ''.join(code)
        cc = {
            'imports': code_imports,
            target: code,
            'callbacks': code_callbacks,
            'tkvariables': list(self._tkvariables.keys())
            }
        return cc
    
    def _toplevel_or_tk(self, target):
        wmeta = self.uidefinition.get_widget(target)
        if wmeta.classname == 'tk.Toplevel':
            wmeta.classname = 'pygubudesigner.ToplevelOrTk'
            self.uidefinition.replace_widget(target, wmeta)
    
    def generate(self, uidef, target, **kw):
        self.uidefinition = uidef
        self._process_options(kw)
        
        mastermeta = WidgetMeta('','master')
        builder = BuilderObject(self, mastermeta)
        self._toplevel_or_tk(target)
        wmeta = self.uidefinition.get_widget(target)

        if wmeta is not None:
            self._code_realize(builder, wmeta)
        return self._process_results(target)
    
    def generate_widget_class(self, uidef, target, **kw):
        self.uidefinition = uidef
        self._process_options(kw)
        
        mastermeta = wmeta = self.uidefinition.get_widget(target)
        builder = bmaster = BuilderObject(self, mastermeta)
        if wmeta is not None:
            originalid = wmeta.identifier
            wmeta.identifier = 'self'
            
            if wmeta.classname not in CLASS_MAP:
                self._builder._import_class(wmeta.classname)

            if wmeta.classname in CLASS_MAP:
                bclass = CLASS_MAP[wmeta.classname].builder
                builder = bclass.factory(self, wmeta)
                uniqueid = builder.code_identifier()
                masterid = bmaster.code_child_master()
                
                for childmeta in \
                    self.uidefinition.widget_children(target):
                    childid = self._code_realize(builder, childmeta)
                    code = builder.code_child_add(childid)
                    self._code.extend(code)
        
        return self._process_results(target)
    
    def code_classname_for(self, bobject):
        wmeta = bobject.wmeta
        cname = None
        if bobject.class_ is not None:
            if wmeta.classname == 'pygubudesigner.ToplevelOrTk':
                cname = 'tk.Toplevel'
            elif wmeta.classname.startswith('tk.'):
                cname = wmeta.classname
            elif wmeta.classname.startswith('ttk.'):
                self._import_ttk = True;
                cname = wmeta.classname
            else:
                module = bobject.class_.__module__
                cname = bobject.class_.__name__
                if module not in self._code_imports:
                    self._code_imports[module] = set((cname,))
                else:
                    self._code_imports[module].add(cname)
        return cname
    
    def _process_imports(self):
        lines = []
        s = 'import tkinter as tk'
        lines.append(s)
        if self._import_ttk:
            s = 'import tkinter.ttk as ttk'
            lines.append(s)
        skeys = sorted(self._code_imports.keys())
        for mname in skeys:
            names = sorted(self._code_imports[mname])
            for group in grouper(names, 4):
                bag = []
                for cname in group:
                    if cname is not None:
                        bag.append(cname)
                clist = None
                if len(bag) > 1:
                    clist = '({0})'.format(', '.join(bag))
                else:
                    clist = ''.join(bag)
                line = 'from {0} import {1}'.format(mname, clist)
                lines.append(line)
        return lines
    
    def code_create_variable(self, name_or_desc, value, vtype=None):
        vname, type_from_name = self._process_variable_description(name_or_desc)
        vname_in_code = vname
        if vname not in self._tkvariables:
            var_init = ''
            if value is None:
                value = "''"
            else:
                if type_from_name == 'string':
                    value = "'{0}'".format(value)
            if vtype is None:
                var_init = 'tk.{0}Var(value={1})'.format(
                    type_from_name.capitalize(), value)
            else:
                var_init = '{0}(value={1})'.format(str(vtype), value)
            if self.as_class:
                vname_in_code = 'self.{0}'.format(vname)
            line = '{0} = {1}'.format(vname_in_code, var_init)
            self._code.append(line)
            self._tkvariables[vname] = vname_in_code
        return self._tkvariables[vname]
    
    def _code_realize(self, bmaster, wmeta):
        originalid = wmeta.identifier
        uniqueid = None
        
        if wmeta.classname not in CLASS_MAP:
            self._builder._import_class(wmeta.classname)

        if wmeta.classname in CLASS_MAP:
            bclass = CLASS_MAP[wmeta.classname].builder
            builder = bclass.factory(self, wmeta)
            uniqueid = builder.code_identifier()
            masterid = bmaster.code_child_master()
            
            if self.as_class:
                if not masterid:
                    uniqueid = 'self'
                else:
                    uniqueid = 'self.' + uniqueid
            
            create = builder.code_realize(bmaster, uniqueid)
            self._code.extend(create)
            
            # Children
            for childmeta in \
                self.uidefinition.widget_children(originalid):
                childid = self._code_realize(builder, childmeta)
                code = builder.code_child_add(childid)
                self._code.extend(code)
            
            # configuration
            configure = builder.code_configure()
            self._code.extend(configure)
            
            # layout
            layout = builder.code_layout(parentid=masterid)
            self._code.extend(layout)
            
            # callbacks
            commands = builder.code_connect_commands()
            bindings = builder.code_connect_bindings()
            self._code.extend(commands)
            self._code.extend(bindings)
        else:
            msg = 'Class "{0}" not mapped'.format(wmeta.classname)
            raise Exception(msg)
        
        return uniqueid

    def _process_callbacks(self):
        tabspaces = self._options['tabspaces']
        tab2 = tabspaces//2 if tabspaces == 8 else 1
        
        lines = []
        for name, value in self._callbacks.items():
            wid, cbtype, args = value
            if cbtype == CB_TYPES.BIND_EVENT:
                line = '{0}def {1}(self, event=None):'.format(' '*tab2, name)
                lines.append(line)
                line = '{0}pass\n'.format(' '*tabspaces)
                lines.append(line)
            elif cbtype == CB_TYPES.SCROLL:
                fargs = []
                for a in args:
                    fargs.append('{0}=None'.format(a))
                fargs = ', '.join(fargs)
                line = '{0}def {1}(self, {2}):'.format(' '*tab2, name, fargs)
                lines.append(line)
                line = '{0}pass\n'.format(' '*tabspaces)
                lines.append(line)
            else:
                # other types: cb_simple, cb_with_id, etc.
                if args is None:
                    line = '{0}def {1}(self):'.format(' '*tab2, name)
                    lines.append(line)
                    line = '{0}pass\n'.format(' '*tabspaces)
                    lines.append(line)
                else:
                    fargs = ', '.join(args)
                    line = '{0}def {1}(self, {2}):'.format(' '*tab2, name, fargs)
                    lines.append(line)
                    line = '{0}pass\n'.format(' '*tabspaces)
                    lines.append(line)
        return lines
    
    def code_create_callback(self, widgetid, cbname, cbtype, args=None):
        #print('on_code_create_callback', widgetid, cbname, cbtype, args)
        if cbname not in self._callbacks:
            self._callbacks[cbname] = (widgetid, cbtype, args)
        cb_name = 'self.{0}'.format(cbname)
        return cb_name
    
    def code_create_image(self, filename):
        basename = os.path.basename(filename)
        name, file_ext = os.path.splitext(basename)
        name = self._make_identifier(basename)
        varname = 'self.{0}'.format(name)
            
        if filename not in self._tkimages:
            img_class = 'tk.PhotoImage'
            if file_ext in TK_BITMAP_FORMATS:
                img_class = 'tk.BitmapImage'
            line = "{0} = {1}(file='{2}')".format(varname, img_class, filename)
            self._code.append(line)
            self._tkvariables[varname] = True
        return varname
    
    def code_create_iconbitmap(self, filename):
        TPL = '@{0}'
        if os.name == 'nt':
            TPL = '{0}'
        return TPL.format(filename)
    
    def _make_identifier(self, name):
        output = name.replace('.', '_')
        output = ''.join(x for x in output if x.isalnum() or x == '_')
        return output



# encoding: UTF-8
#
# Copyright 2012-2021 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
from collections import OrderedDict
import json

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubu.builder.builderobject import CB_TYPES
from pygubudesigner.i18n import translator as _
from pygubudesigner.widgets import EntryPropertyEditor
from pygubudesigner.widgets.propertyeditor import register_editor
from pygubudesigner.widgets.commandentry import CommandPropertyBase


class EntryValidateCommandPropertyEditor(CommandPropertyBase):
    
    def _create_ui(self):
        self._cbname = w = EntryPropertyEditor(self)
        w.grid(row=0, column=0, sticky='nswe', columnspan=2)
        w.bind('<<PropertyChanged>>', self._on_variable_changed)
        
        self._arg_var = var = tk.StringVar(self, '')
        self._argd = w = EntryPropertyEditor(self)
        w.parameters(state='readonly', textvariable=var)
        w.grid(row=1, column=0, sticky='nswe')
        
        self._mb = w = ttk.Menubutton(self, text='Args')
        w.grid(row=1, column=1, sticky='nswe')
        
        olist = (
            ('%d', _('Type of action.')),
            ('%i', _('Index of character string to be inserted/deleted.')),
            ('%P', _("The new/current value of the entry.")),
            ('%s', _('The current value of entry prior to editing.')),
            ('%S', _('The text string being inserted/deleted.')),
            ('%v', _('The current value of the -validate option.')),
            ('%V', _('The validation condition that triggered the callback.')),
            ('%W', _('The name of the entry widget.')),
        )
        options = OrderedDict(olist)
        self._vars = { k: tk.BooleanVar(self) for k in options.keys() }
        
        self._menu = m = tk.Menu(self._mb, tearoff=False)
        m.add_command(label=_('Select all'), command=self._on_select_all)
        m.add_command(label=_('Clear'), command=self._on_clear)
        m.add_separator()
        for key, txt in options.items():
            txt = '{0} {1}'.format(key, txt)
            m.add_checkbutton(label=txt, variable=self._vars[key],
                              command=self._on_update_args)
        self._mb['menu'] = m
        self.columnconfigure(0, weight=1)
    
    def _on_select_all(self):
        for key, var in self._vars.items():
            var.set(True)
        self._on_update_args()
        self._on_variable_changed()
    
    def _on_clear(self):
        for key, var in self._vars.items():
            var.set(False)
        self._on_update_args()
        self._on_variable_changed()
    
    def _on_update_args(self, event=None):
        self._arg_var.set(self._get_args())
        self._on_variable_changed()
    
    def _get_args(self):
        args = []
        for key, var in self._vars.items():
            if var.get():
                args.append(key)
        args = ' '.join(args)
        return args
    
    def _set_args(self, args):
        arglist = args.split(' ')
        for key, var in self._vars.items():
            var.set(False)
            if key in arglist:
                var.set(True)
    
    def _set_value(self, value):
        """Save value on storage"""
        cbname = ''
        args = ''
        if len(value) != 0:
            vd = json.loads(value)
            cbname = vd['value']
            args = vd['args']
        self._cbname.edit(cbname)
        self._set_args(args)
        self._arg_var.set(self._get_args())
        self._variable.set(value)
    
    def _get_value(self):
        value = ''
        if len(self._cbname.value) != 0:
            args = self._get_args()
            cmd = {
                'type': 'command',
                'value': self._cbname.value,
                'cbtype': CB_TYPES.ENTRY_VALIDATE,
                'args': args,
            }
            value = json.dumps(cmd)
        return value
    
    def _validate(self):
        is_valid = True
        value = self._cbname.value
        if len(value) != 0:
            is_valid = self.is_safe_identifier(value)
        self.show_invalid(not is_valid)
        return is_valid

register_editor('entryvalidatecommandentry', EntryValidateCommandPropertyEditor)


if __name__ == '__main__':
    root = tk.Tk()
    root.columnconfigure(0, weight=1)
    root.rowconfigure(0, weight=1)
    
    editor = EntryValidateCommandPropertyEditor(root)
    editor.grid(sticky='nsew')
    
    def on_change_cb(self, event=None):
        print('change cb')
        print(editor.value)
    
    editor.bind('<<PropertyChanged>>', on_change_cb)
    value = '{"value":"mycb", "cbtype": "entryvalidate"}'
    editor.edit(value)
    
    root.mainloop()


# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here
from __future__ import unicode_literals, print_function
import logging

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubu import builder
from pygubu.widgets.simpletooltip import create as create_tooltip
from pygubudesigner.widgets.propertyeditor import create_editor
from pygubudesigner import properties
from pygubudesigner.i18n import translator as _

logger = logging.getLogger(__name__)
CLASS_MAP = builder.CLASS_MAP


class PropertiesEditor(object):
    def __init__(self, frame, **kw):
        self._current = None
        self._sframe = frame
        self._frame = None
        self._propbag = {}
        self._id_validator = kw.get('id_validator', None)
        self._create_properties()
        self.hide_all()

    def _create_properties(self):
        """Populate a frame with a list of all editable properties"""
        self._frame = f = ttk.Labelframe(self._sframe.innerframe,
                                         text=_('Widget properties'))
        f.configure(padding=4)
        f.grid(sticky='nswe')

        label_tpl = "{0}:"
        row = 0
        col = 0

        groups = (
            ('00', _('Required'), properties.WIDGET_REQUIRED_OPTIONS,
             properties.REQUIRED_OPTIONS),
            ('01', _('Standard'), properties.WIDGET_STANDARD_OPTIONS,
             properties.TK_WIDGET_OPTIONS),
            ('02', _('Specific'), properties.WIDGET_SPECIFIC_OPTIONS,
             properties.TK_WIDGET_OPTIONS),
            ('03', _('Custom'), properties.WIDGET_CUSTOM_OPTIONS,
             properties.CUSTOM_OPTIONS),
        )

        for gcode, gname, plist, propdescr in groups:
            padding = '0 0 0 5' if row == 0 else '0 5 0 5'
            label = ttk.Label(self._frame, text=gname,
                              font='TkDefaultFont 10 bold', padding=padding,
                              foreground='#000059')
            label.grid(row=row, column=0, sticky='we', columnspan=2)
            row += 1
            for name in plist:
                kwdata = propdescr[name]
                labeltext = label_tpl.format(name)
                label = ttk.Label(self._frame, text=labeltext, anchor=tk.W)
                label.grid(row=row, column=col, sticky=tk.EW, pady=2)
                label.tooltip = create_tooltip(label, '?')
                widget = self._create_editor(self._frame, name, kwdata)
                widget.grid(row=row, column=col+1, sticky=tk.EW, pady=2)
                row += 1
                self._propbag[gcode+name] = (label, widget)
                logger.debug('Created property: %s-%s', gname,name)
                # add validator for widget id
                if gcode == '00' and name == 'id':
                    widget.set_unique_cb(self._id_validator)

    def _create_editor(self, master, pname, wdata):
        editor = None
        wtype = wdata.get('editor', None)

        #I don't have class name at this moment
        #so setup class specific values on update_property_widget
        editor = create_editor(wtype, master)

        def make_on_change_cb(pname, editor):
            def on_change_cb(event):
                self._on_property_changed(pname, editor)
            return on_change_cb

        editor.bind('<<PropertyChanged>>', make_on_change_cb(pname, editor))
        return editor

    def _on_property_changed(self, name, editor):
        self._current.widget_property(name, editor.value)

    def update_editor(self, label, editor, wdescr, pname, propdescr):
        pdescr = propdescr.copy()
        classname = wdescr.classname

        if classname in pdescr:
            pdescr = dict(pdescr, **pdescr[classname])

        params = pdescr.get('params', {})
        editor.parameters(**params)
        default = pdescr.get('default', '')
        
        help = pdescr.get('help', None)
        if isinstance(help, dict):
            for k, v in help.items():
                if classname.startswith(k):
                    help = v
                    break
        label.tooltip.text = help

        value = wdescr.widget_property(pname)
        if not value and default:
            value = default
        editor.edit(value)

    def edit(self, wdescr):
        self._current = wdescr
        wclass = wdescr.classname
        class_descr = CLASS_MAP[wclass].builder
        
        # GroupCode, PropertyType, dict, tuple
        groups = (
            ('00', None, properties.WIDGET_REQUIRED_OPTIONS,
             properties.REQUIRED_OPTIONS),
            ('01', 'OPTIONS_STANDARD', properties.WIDGET_STANDARD_OPTIONS,
             properties.TK_WIDGET_OPTIONS),
            ('02', 'OPTIONS_SPECIFIC', properties.WIDGET_SPECIFIC_OPTIONS,
             properties.TK_WIDGET_OPTIONS),
            ('03', 'OPTIONS_CUSTOM', properties.WIDGET_CUSTOM_OPTIONS,
             properties.CUSTOM_OPTIONS)
        )
        for gcode, attrname, proplist, gproperties in groups:
            for name in proplist:
                propdescr = gproperties[name]
                label, widget = self._propbag[gcode + name]
                if gcode == '00' or name in getattr(class_descr, attrname):
                    self.update_editor(label, widget, wdescr, name, propdescr)
                    label.grid()
                    widget.grid()
                else:
                    #hide property widget
                    label.grid_remove()
                    widget.grid_remove()
        self._sframe.reposition()

    def hide_all(self):
        """Hide all properties from property editor."""
        self.current = None

        for _v, (label, widget) in self._propbag.items():
            label.grid_remove()
            widget.grid_remove()



# encoding: utf-8


class LogPanelManager(object):
    def __init__(self, app):
        self.app = app
        builder = self.app.builder
        self.btn_messages = builder.get_object('btn_messages')
        self.txt_log = builder.get_object('txt_log')
        self.buttonsvar = builder.get_variable('bpanel_buttonsvar')
        self.bpanel = builder.get_object('bpanel')
        self.gcontainer = builder.get_object('bp_container')
        self.gbuttons = builder.get_object('bp_buttons')
        self.mainpw = builder.get_object('mainpw')
        self.mainpw.bind('<Configure>', self.pwindow_configure)
        self.mainpw_sash_pos = None
        self.gcontainer.pack_forget()
        self.btn_messages_label = self.btn_messages.cget('text')
        self.unread = 0

    def pwindow_configure(self, event):
        self.mainpw_sash_pos = None
        
    def update_sash(self, action='show'):
        pwh = self.mainpw.winfo_height()
        gch = self.bpanel.winfo_reqheight()
        gbh = self.gbuttons.winfo_height()
        newpos = pwh-gch
        hidden_pos = pwh - (gbh+5)
        if action == 'show':
            if self.mainpw_sash_pos is not None:
                if self.mainpw_sash_pos > hidden_pos:
                    self.mainpw_sash_pos = newpos
                newpos = self.mainpw_sash_pos
        elif action == 'hide':
            newpos = hidden_pos
            self.mainpw_sash_pos = self.mainpw.sashpos(0)
        self.mainpw.sashpos(0, newpos)
        
    def on_bpanel_button_clicked(self):
        #print(self.pwindow.sashpos(0))
        value = self.buttonsvar.get()
        if value == 'messages':
            self.unread = 0
            self.btn_messages.config(text=self.btn_messages_label)
            self.gcontainer.pack(side='top', expand=True, fill='both')
            self.gcontainer.after_idle(self.update_sash)
        else:
            self.gcontainer.after_idle(
                lambda x='hide':self.update_sash(x))
            self.gcontainer.pack_forget()
    
    def _log_set_text(self, text):
        tktext = self.txt_log
        tktext.config(state='normal')
        tktext.delete('0.0', 'end')
        tktext.insert('0.0', text)
        tktext.config(state='disabled')
    
    def _log_add_text(self, text):
        tktext = self.txt_log
        tktext.config(state='normal')
        tktext.insert('end', text+'\n')
        tktext.config(state='disabled')
        tktext.see('end')
    
    def log_message(self, msg, level):
        if self.buttonsvar.get() != 'messages':
            self.unread = self.unread + 1
            # log panel is not visible, update label
            label = '{0} ({1})'.format(self.btn_messages_label, self.unread)
            self.btn_messages.config(text=label)
        self._log_add_text(msg)



# encoding: UTF-8
# This file is part of pygubu.

#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
from __future__ import print_function
import platform
import os
import sys
import logging
import webbrowser
import importlib
import argparse

try:
    import tkinter as tk
    from tkinter import ttk
    from tkinter import filedialog
    from tkinter import messagebox
except:
    import Tkinter as tk
    import ttk
    import tkMessageBox as messagebox
    import tkFileDialog as filedialog


import pygubu
from pygubu import builder
from pygubu.stockimage import StockImage, StockImageException
from .util import virtual_event, menu_iter_children, get_ttk_style
from .util.keyboard import Key, key_bind
from .uitreeeditor import WidgetsTreeEditor
from .previewer import PreviewHelper
from .i18n import translator
import pygubudesigner
from pygubudesigner import preferences as pref
from pygubudesigner.widgets.componentpalette import ComponentPalette
from pygubudesigner.widgets.toolbarframe import ToolbarFrame
from pygubudesigner.scriptgenerator import ScriptGenerator
from pygubudesigner.dialogs import ask_save_changes, AskSaveChangesDialog
from .rfilemanager import RecentFilesManager
from .logpanel import LogPanelManager
import pygubudesigner.actions as actions


#Initialize logger
logger = logging.getLogger(__name__)

#translator function
_ = translator

def init_pygubu_widgets():
    import pkgutil
    #initialize standard ttk widgets
    import pygubu.builder.ttkstdwidgets

    #initialize extra widgets
    widgets_pkg = 'pygubu.builder.widgets'
    mwidgets = importlib.import_module(widgets_pkg)

    for x, modulename, x in pkgutil.iter_modules(mwidgets.__path__, mwidgets.__name__ + "."):
        try:
            importlib.import_module(modulename)
        except Exception as e:
            logger.exception(e)
            msg = _("Failed to load widget module: '{0}'".format(modulename))
            messagebox.showerror(_('Error'), msg)

    #initialize custom widgets
    for path in pref.get_custom_widgets():
        dirname, fname = os.path.split(path)
        if fname.endswith('.py'):
            if dirname not in sys.path:
                sys.path.append(dirname)
            modulename = fname[:-3]
            try:
                importlib.import_module(modulename)
            except Exception as e:
                logger.exception(e)
                msg = _("Failed to load custom widget module: '{0}'".format(path))
                messagebox.showerror(_('Error'), msg)

#Initialize images
DESIGNER_DIR = os.path.dirname(os.path.abspath(__file__))

imgformat = 'images-gif'
if tk.TkVersion >= 8.6:
    imgformat = 'images-png'

IMAGES_DIR = os.path.join(DESIGNER_DIR, "images")
IMAGE_PATHS = [ #(dir, tag)
    (IMAGES_DIR, ''),
    (os.path.join(IMAGES_DIR, imgformat), ''),
    (os.path.join(IMAGES_DIR, imgformat, 'widgets', '22x22'), '22x22-'),
    (os.path.join(IMAGES_DIR, imgformat, 'widgets', '16x16'), '16x16-'),
    (os.path.join(IMAGES_DIR, imgformat, 'widgets', 'fontentry'), ''),
    ]
for dir_, prefix in IMAGE_PATHS:
    StockImage.register_from_dir(dir_, prefix)



class StatusBarHandler(logging.Handler):
    def __init__(self, app, level=logging.NOTSET):
        super(StatusBarHandler, self).__init__(level)
        self.app = app
        formatter = logging.Formatter('%(asctime)s %(levelname)s:%(message)s', datefmt='%H:%M:%S')
        self.setFormatter(formatter)

    def emit(self, record):
        try:
            msg = self.format(record)
            self.app.log_message(msg, record.levelno)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


FILE_PATH = os.path.dirname(os.path.abspath(__file__))


class PygubuDesigner(object):
    """Main gui class"""
    
    def __init__(self):
        """Creates all gui widgets"""

        self.translator = translator
        self.preview = None
        self.about_dialog = None
        self.preferences = None
        self.script_generator = None
        self.builder = pygubu.Builder(translator)
        self.currentfile = None
        self.is_changed = False
        self.current_title = 'new'

        uifile = os.path.join(FILE_PATH, "ui/pygubu-ui.ui")
        self.builder.add_from_file(uifile)
        self.builder.add_resource_path(os.path.join(FILE_PATH, "images"))

        in_macos = (sys.platform == 'darwin')
        #build main ui
        self.mainwindow = self.builder.get_object('mainwindow')
        menu = self.builder.get_object('mainmenu', self.mainwindow)
        
        if in_macos:
            cmd = 'tk::mac::ShowPreferences'
            self.mainwindow.createcommand(cmd, self._edit_preferences)
            #cmd = 'tk::mac::ShowHelp'
            #self.mainwindow.createcommand(cmd, self.on_help_item_clicked)
            cmd = 'tk::mac::Quit'
            self.mainwindow.createcommand(cmd, self.quit)
            # In mac add apple menu
            m = tk.Menu(menu, name='apple')
            m.add_command(label=_('Quit …'), accelerator='Cmd-Q',
                          command=self.quit)
            menu.insert_cascade(0, menu=m)
        
        # Set top menu
        self.mainwindow.configure(menu=menu)

        # Recen Files management
        rfmenu = self.builder.get_object('file_recent_menu')
        self.rfiles_manager = RecentFilesManager(rfmenu, self.do_file_open)
        self.mainwindow.after_idle(lambda: self.rfiles_manager.load())

        #widget tree
        self.treeview = self.builder.get_object('treeview1')
        self.bindings_frame = self.builder.get_object('bindingsframe')
        self.bindings_tree = self.builder.get_object('bindingstree')
        
        # Load all widgets before creating the component pallete
        init_pygubu_widgets()
        
        # _pallete
        self.fpalette = self.builder.get_object('fpalette')
        self.create_component_palette(self.fpalette)

        #Preview
        previewc = self.builder.get_object('preview_canvas')
        self.previewer = PreviewHelper(previewc)
        #tree editor
        self.tree_editor = WidgetsTreeEditor(self)
        
        # Tab Code
        self.script_generator = ScriptGenerator(self)
        
        # Bottom Panel
        self.setup_bottom_panel()

        self.builder.connect_callbacks(self)

        #
        # Application Keyboard bindings
        #
        CONTROL_KP_SEQUENCE = '<Control-KeyPress>'
        ALT_KP_SEQUENCE = '<Alt-KeyPress>'
        if in_macos:
            CONTROL_KP_SEQUENCE = '<Command-KeyPress>'
            ALT_KP_SEQUENCE = '<Control-KeyPress>'
            # Fix menu accelerators
            for m, itemtype, index in menu_iter_children(menu):
                if itemtype != 'separator':
                    accel = m.entrycget(index, 'accelerator')
                    accel = accel.replace('Ctrl+', 'Cmd-')
                    accel = accel.replace('Alt+', 'Ctrl+')
                    m.entryconfigure(index, accelerator=accel)
        
        
        master = self.mainwindow
        master.bind_all(CONTROL_KP_SEQUENCE,
                        key_bind(Key.N,
                                     virtual_event(actions.FILE_NEW)))
        master.bind_all(CONTROL_KP_SEQUENCE,
                        key_bind(Key.O,
                                     virtual_event(actions.FILE_OPEN)),
                        add=True)
        master.bind_all(CONTROL_KP_SEQUENCE,
                        key_bind(Key.S,
                                     virtual_event(actions.FILE_SAVE)),
                        add=True)
        master.bind_all(CONTROL_KP_SEQUENCE,
                        key_bind(Key.Q,
                                     virtual_event(actions.FILE_QUIT)),
                        add=True)
        master.bind_all(
            '<F5>',
            virtual_event(actions.TREE_ITEM_PREVIEW_TOPLEVEL))
        master.bind_all(
            '<F6>',
            virtual_event(actions.PREVIEW_TOPLEVEL_CLOSE_ALL))

        # Tree Editing Keyboard events
        for widget in (self.treeview, previewc):
            widget.bind(
                CONTROL_KP_SEQUENCE,
                key_bind(Key.I,
                             virtual_event(actions.TREE_ITEM_MOVE_UP)))
            widget.bind(
                CONTROL_KP_SEQUENCE,
                key_bind(Key.K,
                             virtual_event(actions.TREE_ITEM_MOVE_DOWN)),
                add=True
            )
            widget.bind(
                CONTROL_KP_SEQUENCE,
                key_bind(Key.C,
                             lambda e: self.tree_editor.copy_to_clipboard()),
                add=True)
            widget.bind(
                CONTROL_KP_SEQUENCE,
                key_bind(Key.V,
                             lambda e: self.tree_editor.paste_from_clipboard()),
                add=True
            )
            widget.bind(
                CONTROL_KP_SEQUENCE,
                key_bind(Key.X,
                             lambda e: self.tree_editor.cut_to_clipboard()),
                add=True
            )
            widget.bind('<KeyPress>',
                        key_bind(Key.I,
                                     virtual_event(actions.TREE_NAV_UP)))
            widget.bind('<KeyPress>',
                        key_bind(Key.K,
                                     virtual_event(actions.TREE_NAV_DOWN)),
                        add=True)
            # avoid double delete dialog in macos
            if not in_macos:
                widget.bind('<KeyPress-Delete>',
                            virtual_event(actions.TREE_ITEM_DELETE))

            #grid move bindings
            widget.bind(
                ALT_KP_SEQUENCE,
                key_bind(Key.I,
                             virtual_event(actions.TREE_ITEM_GRID_UP)))
            widget.bind(
                ALT_KP_SEQUENCE,
                key_bind(Key.K,
                             virtual_event(actions.TREE_ITEM_GRID_DOWN)), add=True)
            widget.bind(
                ALT_KP_SEQUENCE,
                key_bind(Key.J,
                             virtual_event(actions.TREE_ITEM_GRID_LEFT)), add=True)
            widget.bind(
                ALT_KP_SEQUENCE,
                key_bind(Key.L,
                             virtual_event(actions.TREE_ITEM_GRID_RIGHT)), add=True)
        
        # Actions Bindings
        w = self.mainwindow
        w.bind(actions.FILE_NEW, self.on_file_new)
        w.bind(actions.FILE_OPEN, lambda e: self.do_file_open())
        w.bind(actions.FILE_SAVE, self.on_file_save)
        w.bind(actions.FILE_SAVEAS, lambda e: self.do_save_as())
        w.bind(actions.FILE_QUIT, lambda e: self.quit())
        w.bind(actions.FILE_RECENT_CLEAR, lambda e: self.rfiles_manager.clear())
        # On preferences save binding
        w.bind('<<PygubuDesignerPreferencesSaved>>', self.on_preferences_saved)
        
        # setup app preferences
        self.setup_app_preferences()

        #
        # Setup tkk styles
        #
        self._setup_styles()

        #
        # Setup dynamic theme submenu
        #
        self._setup_theme_menu()

        # App config
        top = self.mainwindow
        try:
            top.wm_iconname('pygubu')
            top.tk.call('wm', 'iconphoto', '.', StockImage.get('pygubu'))
        except StockImageException as e:
            pass
        
    def run(self):
        self.mainwindow.protocol("WM_DELETE_WINDOW", self.__on_window_close)
        self.mainwindow.mainloop()
    
    def __on_window_close(self):
        """Manage WM_DELETE_WINDOW protocol."""
        if self.on_close_execute():
            self.mainwindow.withdraw()
            self.mainwindow.destroy()

    def quit(self):
        """Exit the app if it is ready for quit."""
        self.__on_window_close()
        
    def set_title(self, newtitle):
        self.current_title = newtitle
        default_title = 'Pygubu Designer - {0}'
        title = default_title.format(newtitle)
        self.mainwindow.wm_title(title)

    def _setup_styles(self):
        self.mainwindow.option_add('*Dialog.msg.width', 34)
        self.mainwindow.option_add("*Dialog.msg.wrapLength", "6i")
        
        s = get_ttk_style()
        s.configure('ColorSelectorButton.Toolbutton',
                    image=StockImage.get('mglass'))
        s.configure('ImageSelectorButton.Toolbutton',
                    image=StockImage.get('mglass'))
        s.configure('ComponentPalette.Toolbutton',
                    font='TkSmallCaptionFont')
        s.configure('ComponentPalette.TNotebook.Tab',
                    font='TkSmallCaptionFont')
        s.configure('PanelTitle.TLabel',
                    background='#808080',
                    foreground='white',
                    font='TkSmallCaptionFont')
        s.configure('Template.Toolbutton',
                    padding=5)
        # ToolbarFrame scroll buttons
        s.configure(ToolbarFrame.BTN_LEFT_STYLE,
            image=StockImage.get('arrow-left2'))
        s.configure(ToolbarFrame.BTN_RIGHT_STYLE,
            image=StockImage.get('arrow-right2'))
        if sys.platform == 'linux':
            #change background of comboboxes
            color = s.lookup('TEntry', 'fieldbackground')
            s.map('TCombobox', fieldbackground=[('readonly', color)])
            s.map('TSpinbox', fieldbackground=[('readonly', color)])

    def _setup_theme_menu(self):
        menu = self.builder.get_object('preview_themes_submenu')
        s = get_ttk_style()
        styles = list(s.theme_names())
        styles.sort()
        self.__theme_var = var = tk.StringVar()
        theme = pref.get_option('ttk_theme')
        var.set(theme)

        for name in styles:

            def handler(theme=name):
                self._change_ttk_theme(theme)

            menu.add_radiobutton(label=name, value=name,
                                 variable=self.__theme_var, command=handler)

    def create_treelist(self):
        root_tagset = set(('tk', 'ttk'))

        #create unique tag set
        tagset = set()
        for c in builder.CLASS_MAP.keys():
            wc = builder.CLASS_MAP[c]
            tagset.update(wc.tags)
        tagset.difference_update(root_tagset)

        treelist = []
        for c in builder.CLASS_MAP.keys():
            wc = builder.CLASS_MAP[c]
            ctags = set(wc.tags)
            roots = (root_tagset & ctags)
            sections = (tagset & ctags)
            for r in roots:
                for s in sections:
                    key = '{0}>{1}'.format(r, s)
                    treelist.append((key, wc))

        #sort tags by label
        def by_label(t):
            return "{0}{1}".format(t[0], t[1].label)
        treelist.sort(key=by_label)
        return treelist
    
    def create_component_palette(self, fpalette):
        #Default widget image:
        default_image = ''
        try:
            default_image = StockImage.get('22x22-tk.default')
        except StockImageException as e:
            pass
        
        treelist = self.create_treelist()
        self._pallete = ComponentPalette(fpalette)
        
        #Start building widget tree selector
        roots = {}
        sections = {}
        for key, wc in treelist:
            root, section = key.split('>')
            if section not in sections:
                roots[root] = self._pallete.add_tab(section, section)
                sections[section] = 1
            
            #insert widget
            w_image = default_image
            try:
                w_image = StockImage.get('22x22-{0}'.format(wc.classname))
            except StockImageException as e:
                pass

            #define callback for button
            def create_cb(cname):
                return lambda: self.on_add_widget_event(cname)
            
            wlabel = wc.label
            if wlabel.startswith('Menuitem.'):
                wlabel = wlabel.replace('Menuitem.', '')
            callback = create_cb(wc.classname)
            self._pallete.add_button(section, root, wlabel, wc.classname,
                                     w_image, callback)
        default_group = pref.get_option('widget_set')
        self._pallete.show_group(default_group)

    def on_add_widget_event(self, classname):
        "Adds a widget to the widget tree."""

        self.tree_editor.add_widget(classname)
        self.tree_editor.treeview.focus_set()
    
    def setup_app_preferences(self):
        # Restore windows position and size
        geom = pref.get_window_size()
        self.mainwindow.geometry(geom)
        # Load preferred ttk theme
        theme = pref.get_option('ttk_theme')
        self._change_ttk_theme(theme)
    
    def _change_ttk_theme(self, theme):
        s = get_ttk_style()
        try:
            s.theme_use(theme)
            self._setup_styles()
        except tk.TclError as e:
            logger.exception('Invalid ttk theme.')
    
    def on_preferences_saved(self, event=None):
        # Setup ttk theme if changed
        theme = pref.get_option('ttk_theme')
        self._change_ttk_theme(theme)
    
    def ask_save_changes(self, message, detail=None, title=None):
        do_continue = False
        if title is None:
            title = _('Save Changes')
        if detail is None:
            detail = _("If you don't save the document, all the changes will be lost.")
        choice = ask_save_changes(self.mainwindow, title, message, detail)
        if choice == AskSaveChangesDialog.SAVE:
            do_continue = self.on_file_save()
        elif choice == AskSaveChangesDialog.CANCEL:
            do_continue = False
        elif choice == AskSaveChangesDialog.DONTSAVE:
            do_continue = True
        return do_continue
    
    def on_close_execute(self):
        quit = True
        if self.is_changed:
            msg = _('Do you want to save the changes before closing?')
            quit = self.ask_save_changes(msg)
        if quit:
            # prevent tk image errors on python2 ?
            StockImage.clear_cache()
            
            # Save window size and position
            geom = self.mainwindow.geometry()
            pref.save_window_size(geom)
        return quit

    def do_save(self, fname):
        saved = False
        try:
            self.save_file(fname)
            self.set_changed(False)
            logger.info(_('Project saved to %s'), fname)
            saved = True
        except Exception as e:
            messagebox.showerror(_('Error'), str(e), parent=self.mainwindow)
        return saved

    def do_save_as(self):
        saved = False
        options = {
            'defaultextension': '.ui',
            'filetypes': ((_('pygubu ui'), '*.ui'), (_('All'), '*.*'))}
        fname = filedialog.asksaveasfilename(**options)
        if fname:
            saved = self.do_save(fname)
        return saved

    def save_file(self, filename):
        uidefinition = self.tree_editor.tree_to_uidef()
        uidefinition.save(filename)
        self.currentfile = filename
        title = self.project_name()
        self.set_title(title)
        self.rfiles_manager.addfile(filename)

    def set_changed(self, newvalue=True):
        if newvalue and self.is_changed == False:
            self.set_title('{0} (*)'.format(self.current_title))
        self.is_changed = newvalue

    def load_file(self, filename):
        """Load xml into treeview"""

        try:
            self.tree_editor.load_file(filename)
            self.currentfile = filename
            title = self.project_name()
            self.set_title(title)
            self.set_changed(False)
            self.rfiles_manager.addfile(filename)
        except Exception as e:
            messagebox.showerror(_('Error'), str(e), parent=self.mainwindow)
    
    def do_file_open(self, filename=None):
        openfile = True
        if self.is_changed:
            msg = _('Do you want to save the changes before opening?')
            openfile = self.ask_save_changes(msg)
        if openfile:
            if filename is None:
                options = {
                    'defaultextension': '.ui',
                    'filetypes': ((_('pygubu ui'), '*.ui'), (_('All'), '*.*'))}
                filename = filedialog.askopenfilename(**options)
            if filename:
                self.load_file(filename)
    
    def on_file_new(self, event=None):
        new = True
        if self.is_changed:
            msg = _('Do you want to save the changes before creating?')
            new = self.ask_save_changes(msg)
        if new:
            self.previewer.remove_all()
            self.tree_editor.remove_all()
            self.currentfile = None
            self.set_changed(False)
            self.set_title(self.project_name())
            self.script_generator.reset()
    
    def on_file_save(self, event=None):
        if self.currentfile:
            if self.is_changed:
                return self.do_save(self.currentfile)
        else:
            return self.do_save_as()

    #File Menu
    def on_file_menuitem_clicked(self, itemid):
        action = '<<ACTION_{0}>>'.format(itemid)
        self.mainwindow.event_generate(action)

    #Edit menu
    def on_edit_menuitem_clicked(self, itemid):
        if itemid == 'edit_preferences':
            self._edit_preferences()
        else:
            action = '<<ACTION_{0}>>'.format(itemid)
            self.mainwindow.event_generate(action)

    #preview menu
    def on_previewmenu_action(self, itemid):
        action = '<<ACTION_{0}>>'.format(itemid)
        self.mainwindow.event_generate(action)

    #Help menu
    def on_help_menuitem_clicked(self, itemid):
        if itemid == 'help_online':
            url = 'https://github.com/alejandroautalan/pygubu/wiki'
            webbrowser.open_new_tab(url)
        elif itemid == 'help_about':
            self.show_about_dialog()

    def _create_about_dialog(self):
        builder = pygubu.Builder(translator)
        uifile = os.path.join(FILE_PATH, "ui/about_dialog.ui")
        builder.add_from_file(uifile)
        builder.add_resource_path(os.path.join(FILE_PATH, "images"))

        dialog = builder.get_object(
            'aboutdialog', self.mainwindow)
        entry = builder.get_object('version')
        txt = entry.cget('text')
        txt = txt.replace('%version%', str(pygubu.__version__))
        entry.configure(text=txt)
        entry = builder.get_object('designer_version')
        txt = entry.cget('text')
        txt = txt.replace('%version%', str(pygubudesigner.__version__))
        entry.configure(text=txt)

        def on_ok_execute():
            dialog.close()

        builder.connect_callbacks({'on_ok_execute': on_ok_execute})

        return dialog

    def show_about_dialog(self):
        if self.about_dialog is None:
            self.about_dialog = self._create_about_dialog()
            self.about_dialog.run()
        else:
            self.about_dialog.show()

    def _edit_preferences(self):
        if self.preferences is None:
            self.preferences = pref.PreferencesUI(self.mainwindow, translator)
        self.preferences.dialog.run()
        
    def project_name(self):
        name = None
        if self.currentfile is None:
            name = _('newproject')
        else:
            name = os.path.basename(self.currentfile)
        return name
    
    def nbmain_tab_changed(self, event):
        tab_desing = 0
        tab_code = 1
        nbook = event.widget
        if nbook.index('current') == tab_code:
            self.script_generator.configure()
    
    # Tab code management
    def on_code_generate_clicked(self):
        self.script_generator.on_code_generate_clicked()
    
    def on_code_copy_clicked(self):
        self.script_generator.on_code_copy_clicked()
    
    def on_code_template_changed(self):
        self.script_generator.on_code_template_changed()
    
    def on_code_save_clicked(self):
        self.script_generator.on_code_save_clicked()
        
    def setup_bottom_panel(self):
        self.log_panel = LogPanelManager(self)
        self.setup_logger_handler()
    
    def on_bpanel_button_clicked(self):
        self.log_panel.on_bpanel_button_clicked()
    
    def setup_logger_handler(self):
        #Status bar
        handler = StatusBarHandler(self)
        handler.setLevel(logging.INFO)
        # add handler to the root logger:
        logging.getLogger().addHandler(handler)
    
    def log_message(self, msg, level):
        self.log_panel.log_message(msg, level)


def start_pygubu():
    print("python: {0} on {1}".format(
                platform.python_version(), sys.platform))
    print("pygubu: {0}".format(pygubu.__version__))
    print("pygubu-designer: {0}".format(pygubudesigner.__version__))
    
    # Setup logging level
    parser = argparse.ArgumentParser()
    parser.add_argument('filename', nargs='?')
    parser.add_argument('--loglevel')
    args = parser.parse_args()
    
    loglevel = str(args.loglevel).upper()
    loglevel = getattr(logging, loglevel, logging.WARNING)
    logging.getLogger('').setLevel(loglevel)
    
    #
    # Dependency check
    #
    help = "Hint, If your are using Debian, install package python3-appdirs."
    if sys.version_info < (3,):
        help = "Hint, If your are using Debian, install package python-appdirs."
    check_dependency('appdirs', '1.3', help)
    
    
    #root = tk.Tk()
    #root.withdraw()
    app = PygubuDesigner()
    #root.deiconify()

    filename = args.filename
    if filename is not None:
        app.load_file(filename)

    app.run()


def check_dependency(modulename, version, help_msg=None):
    try:
        module = importlib.import_module(modulename)
        module_version = "<unknown>"
        for attr in ('version', '__version__', 'ver', 'PYQT_VERSION_STR'):
            v = getattr(module, attr, None)
            if v is not None:
                module_version = v
        msg = "Module %s imported ok, version %s"
        logger.info(msg, modulename, module_version)
    except ImportError as e:
        msg = "I can't import module '%s'. You need to have installed '%s' version %s or higher. %s"
        if help_msg is None:
            help_msg = ''
        logger.error(msg, modulename, modulename, version, help_msg)
        sys.exit(-1)


if __name__ == '__main__':
    start_pygubu()



# encoding: UTF-8
#
# Copyright 2012-2013 Alejandro Autalán
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# For further info, check  http://pygubu.web.here

from __future__ import unicode_literals
import sys

try:
    import tkinter as tk
    import tkinter.ttk as ttk
except:
    import Tkinter as tk
    import ttk

from pygubu.stockimage import *


# in-place prettyprint formatter
def indent(elem, level=0):
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i


class ArrayVar(tk.Variable):
    '''A variable that works as a Tcl array variable'''

    _default = {}
    _elementvars = {}

    def __del__(self):
        self._tk.globalunsetvar(self._name)
        for elementvar in self._elementvars:
            del elementvar


    def __setitem__(self, elementname, value):
        if elementname not in self._elementvars:
            v = ArrayElementVar(varname=self._name,
            elementname=elementname, master=self._master)
            self._elementvars[elementname] = v
        self._elementvars[elementname].set(value)


    def __getitem__(self, name):
        if name in self._elementvars:
            return self._elementvars[name].get()
        return None


    def __call__(self, elementname):
        '''Create a new StringVar as an element in the array'''
        if elementname not in self._elementvars:
            v = ArrayElementVar(varname=self._name, elementname=elementname,
                master=self._master)
            self._elementvars[elementname] = v
        return self._elementvars[elementname]


    def set(self, dictvalue):
        # this establishes the variable as an array
        # as far as the Tcl interpreter is concerned
        self._master.eval("array set {%s} {}" % self._name)

        for (k, v) in dictvalue.items():
            self._tk.call("array", "set", self._name, k, v)

    #python 3.3 hack
    initialize = set


    def get(self):
        '''Return a dictionary that represents the Tcl array'''
        value = {}
        for (elementname, elementvar) in self._elementvars.items():
            value[elementname] = elementvar.get()
        return value


class ArrayElementVar(tk.StringVar):
    '''A StringVar that represents an element of an array'''

    def __init__(self, varname, elementname, master):
        name = "%s(%s)" % (varname, elementname)
        #python2 hack
        if sys.version_info[0] < 3:
            if isinstance(name, unicode):
                name = name.encode('UTF-8')
        tk.StringVar.__init__(self, master, None, name)


def treeview_print(tree, root=''):
    def tree_print(tree, root):
        children = tree.get_children(root)
        if root != '' and children:
            print(root)
        for item in children:
            tree_print(tree, item)
    tree_print(tree, root)


class BraceMessage:
    '''Helper class to use braces {} in log messages'''
    def __init__(self, fmt, *args, **kwargs):
        self.fmt = fmt
        self.args = args
        self.kwargs = kwargs

    def __str__(self):
        return self.fmt.format(*self.args, **self.kwargs)


# Helper to translate log messages with braces
trlog = BraceMessage


def virtual_event(event_name):
    '''Generate virtual event event_name'''
    
    def virtual_event_gen(event):
        event.widget.event_generate(event_name)
    
    return virtual_event_gen

def menu_iter_children(menu):
    '''Iterates all menu items (including submenus).
    Returns (menu, itemtype, index)'''
    count = menu.index(tk.END)
    if count is not None:
        cascades = []
        for i in range(0, count+1):
            itemtype = menu.type(i)
            if itemtype == 'cascade':
                cascade = menu.nametowidget(menu.entrycget(i, 'menu'))
                cascades.append(cascade)
            yield (menu, itemtype, i)
        for m in cascades:
            for child in menu_iter_children(m):
                yield child

__style = None

def get_ttk_style():
    '''Use ttkthemes if module is installed
    '''
    global __style
    if __style is None:
        try:
            from ttkthemes.themed_style import ThemedStyle
            __style = ThemedStyle()
        except:
            __style = ttk.Style()
    return __style